searchNodes=[{"doc":"Cachex provides a straightforward interface for in-memory key/value storage. Cachex is an extremely fast, designed for caching but also allowing for more general in-memory storage. The main goal of Cachex is achieve a caching implementation with a wide array of options, without sacrificing performance. Internally, Cachex is backed by ETS, allowing for an easy-to-use interface sitting upon extremely well tested tools. Cachex comes with support for all of the following (amongst other things): Time-based key expirations Maximum size protection Pre/post execution hooks Statistics gathering Multi-layered caching/key fallbacks Transactions and row locking Asynchronous write operations Syncing to a local filesystem User command invocation All features are optional to allow you to tune based on the throughput needed. See start_link/2 for further details about how to configure these options and example usage.","ref":"Cachex.html","title":"Cachex","type":"module"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Cachex.html#child_spec/1","title":"Cachex.child_spec/1","type":"function"},{"doc":"Removes all entries from a cache. The returned numeric value will contain the total number of keys removed from the cache. This is equivalent to running size/2 before running the internal clear operation. Examples iex&gt; Cachex . put ( :my_cache , &quot;key&quot; , &quot;value&quot; ) iex&gt; Cachex . get ( :my_cache , &quot;key&quot; ) iex&gt; Cachex . size ( :my_cache ) { :ok , 1 } iex&gt; Cachex . clear ( :my_cache ) { :ok , 1 } iex&gt; Cachex . size ( :my_cache ) { :ok , 0 }","ref":"Cachex.html#clear/2","title":"Cachex.clear/2","type":"function"},{"doc":"Retrieves the number of unexpired records in a cache. Unlike size/2 , this ignores keys which should have expired. Due to this taking potentially expired keys into account, it is far more expensive than simply calling size/2 and should only be used when the distinction is completely necessary. Examples iex&gt; Cachex . put ( :my_cache , &quot;key1&quot; , &quot;value1&quot; ) iex&gt; Cachex . put ( :my_cache , &quot;key2&quot; , &quot;value2&quot; ) iex&gt; Cachex . put ( :my_cache , &quot;key3&quot; , &quot;value3&quot; ) iex&gt; Cachex . count ( :my_cache ) { :ok , 3 }","ref":"Cachex.html#count/2","title":"Cachex.count/2","type":"function"},{"doc":"Decrements an entry in the cache. This will overwrite any value that was previously set against the provided key. Options :initial An initial value to set the key to if it does not exist. This will take place before the decrement call. Defaults to 0. Examples iex&gt; Cachex . put ( :my_cache , &quot;my_key&quot; , 10 ) iex&gt; Cachex . decr ( :my_cache , &quot;my_key&quot; ) { :ok , 9 } iex&gt; Cachex . put ( :my_cache , &quot;my_new_key&quot; , 10 ) iex&gt; Cachex . decr ( :my_cache , &quot;my_new_key&quot; , 5 ) { :ok , 5 } iex&gt; Cachex . decr ( :my_cache , &quot;missing_key&quot; , 5 , initial : 2 ) { :ok , - 3 }","ref":"Cachex.html#decr/4","title":"Cachex.decr/4","type":"function"},{"doc":"Removes an entry from a cache. This will return { :ok, true } regardless of whether a key has been removed or not. The true value can be thought of as &quot;is key no longer present?&quot;. Examples iex&gt; Cachex . put ( :my_cache , &quot;key&quot; , &quot;value&quot; ) iex&gt; Cachex . get ( :my_cache , &quot;key&quot; ) { :ok , &quot;value&quot; } iex&gt; Cachex . del ( :my_cache , &quot;key&quot; ) { :ok , true } iex&gt; Cachex . get ( :my_cache , &quot;key&quot; ) { :ok , nil }","ref":"Cachex.html#del/3","title":"Cachex.del/3","type":"function"},{"doc":"Serializes a cache to a location on a filesystem. This operation will write the current state of a cache to a provided location on a filesystem. The written state can be used alongside the load/3 command to import back in the future. It is the responsibility of the user to ensure that the location is able to be written to, not the responsibility of Cachex. Options :compression Specifies the level of compression to apply when serializing (0-9). This will default to level 1 compression, which is appropriate for most dumps. Using a compression level of 0 will disable compression completely. This will result in a faster serialization but at the cost of higher space. Examples iex&gt; Cachex . dump ( :my_cache , &quot;/tmp/my_default_backup&quot; ) { :ok , true } iex&gt; Cachex . dump ( :my_cache , &quot;/tmp/my_custom_backup&quot; , [ compressed : 0 ] ) { :ok , true }","ref":"Cachex.html#dump/3","title":"Cachex.dump/3","type":"function"},{"doc":"Determines whether a cache contains any entries. This does not take the expiration time of keys into account. As such, if there are any unremoved (but expired) entries in the cache, they will be included in the returned determination. Examples iex&gt; Cachex . put ( :my_cache , &quot;key1&quot; , &quot;value1&quot; ) iex&gt; Cachex . empty? ( :my_cache ) { :ok , false } iex&gt; Cachex . clear ( :my_cache ) iex&gt; Cachex . empty? ( :my_cache ) { :ok , true }","ref":"Cachex.html#empty?/2","title":"Cachex.empty?/2","type":"function"},{"doc":"Executes multiple functions in the context of a cache. This can be used when carrying out several cache operations at once to avoid the overhead of cache loading and jumps between processes. This does not provide a transactional execution, it simply avoids the overhead involved in the initial calls to a cache. For a transactional implementation, please see transaction/3 . To take advantage of the cache context, ensure to use the cache instance provided when executing cache calls. If this is not done you will see zero benefits from using execute/3 . Examples iex&gt; Cachex . put ( :my_cache , &quot;key1&quot; , &quot;value1&quot; ) iex&gt; Cachex . put ( :my_cache , &quot;key2&quot; , &quot;value2&quot; ) iex&gt; Cachex . execute ( :my_cache , fn ( worker ) -&gt; ...&gt; val1 = Cachex . get! ( worker , &quot;key1&quot; ) ...&gt; val2 = Cachex . get! ( worker , &quot;key2&quot; ) ...&gt; [ val1 , val2 ] ...&gt; end ) { :ok , [ &quot;value1&quot; , &quot;value2&quot; ] }","ref":"Cachex.html#execute/3","title":"Cachex.execute/3","type":"function"},{"doc":"Determines whether an entry exists in a cache. This will take expiration times into account, meaning that expired entries will not be considered to exist. Examples iex&gt; Cachex . put ( :my_cache , &quot;key&quot; , &quot;value&quot; ) iex&gt; Cachex . exists? ( :my_cache , &quot;key&quot; ) { :ok , true } iex&gt; Cachex . exists? ( :my_cache , &quot;missing_key&quot; ) { :ok , false }","ref":"Cachex.html#exists?/3","title":"Cachex.exists?/3","type":"function"},{"doc":"Places an expiration time on an entry in a cache. The provided expiration must be a integer value representing the lifetime of the entry in milliseconds. If the provided value is not positive, the entry will be immediately evicted. If the entry does not exist, no changes will be made in the cache. Examples iex&gt; Cachex . put ( :my_cache , &quot;key&quot; , &quot;value&quot; ) iex&gt; Cachex . expire ( :my_cache , &quot;key&quot; , :timer . seconds ( 5 ) ) { :ok , true } iex&gt; Cachex . expire ( :my_cache , &quot;missing_key&quot; , :timer . seconds ( 5 ) ) { :ok , false }","ref":"Cachex.html#expire/4","title":"Cachex.expire/4","type":"function"},{"doc":"Updates an entry in a cache to expire at a given time. Unlike expire/4 this call uses an instant in time, rather than a duration. The same semantics apply as calls to expire/4 in that instants which have passed will result in immediate eviction. Examples iex&gt; Cachex . put ( :my_cache , &quot;key&quot; , &quot;value&quot; ) iex&gt; Cachex . expire_at ( :my_cache , &quot;key&quot; , 1455728085502 ) { :ok , true } iex&gt; Cachex . expire_at ( :my_cache , &quot;missing_key&quot; , 1455728085502 ) { :ok , false }","ref":"Cachex.html#expire_at/4","title":"Cachex.expire_at/4","type":"function"},{"doc":"Exports all entries from a cache. This provides a raw read of the entire backing table into a list of cache records for export purposes. This function is very heavy, so it should typically only be used when debugging and/or exporting of tables (although the latter case should really use dump/3 ). ## Examples iex&gt; Cachex . put ( :my_cache , &quot;key&quot; , &quot;value&quot; ) iex&gt; Cachex . export ( :my_cache ) { :ok , [ { :entry , &quot;key&quot; , 1538714590095 , nil , &quot;value&quot; } ] }","ref":"Cachex.html#export/2","title":"Cachex.export/2","type":"function"},{"doc":"Fetches an entry from a cache, generating a value on cache miss. If the entry requested is found in the cache, this function will operate in the same way as get/3 . If the entry is not contained in the cache, the provided fallback function will be executed. A fallback function is a function used to lazily generate a value to place inside a cache on miss. Consider it a way to achieve the ability to create a read-through cache. A fallback function should return a Tuple consisting of a :commit or :ignore tag and a value. If the Tuple is tagged :commit the value will be placed into the cache and then returned. If tagged :ignore the value will be returned without being written to the cache. If you return a value which does not fit this structure, it will be assumed that you are committing the value. If a fallback function has an arity of 1, the requested entry key will be passed through to allow for contextual computation. If a function has an arity of 2, the :provide option from the global :fallback cache option will be provided as the second argument. This is to allow easy state sharing, such as remote clients. If a function has an arity of 0, it will be executed without arguments. If a cache has been initialized with a default fallback function in the :fallback option at cache startup, the third argument to this call becomes optional. Examples iex&gt; Cachex . put ( :my_cache , &quot;key&quot; , &quot;value&quot; ) iex&gt; Cachex . fetch ( :my_cache , &quot;key&quot; , fn ( key ) -&gt; ...&gt; { :commit , String . reverse ( key ) } ...&gt; end ) { :ok , &quot;value&quot; } iex&gt; Cachex . fetch ( :my_cache , &quot;missing_key&quot; , fn ( key ) -&gt; ...&gt; { :ignore , String . reverse ( key ) } ...&gt; end ) { :ignore , &quot;yek_gnissim&quot; } iex&gt; Cachex . fetch ( :my_cache , &quot;missing_key&quot; , fn ( key ) -&gt; ...&gt; { :commit , String . reverse ( key ) } ...&gt; end ) { :commit , &quot;yek_gnissim&quot; }","ref":"Cachex.html#fetch/4","title":"Cachex.fetch/4","type":"function"},{"doc":"Retrieves an entry from a cache. Examples iex&gt; Cachex . put ( :my_cache , &quot;key&quot; , &quot;value&quot; ) iex&gt; Cachex . get ( :my_cache , &quot;key&quot; ) { :ok , &quot;value&quot; } iex&gt; Cachex . get ( :my_cache , &quot;missing_key&quot; ) { :ok , nil }","ref":"Cachex.html#get/3","title":"Cachex.get/3","type":"function"},{"doc":"Retrieves and updates an entry in a cache. This operation can be seen as an internal mutation, meaning that any previously set expiration time is kept as-is. This function accepts the same return syntax as fallback functions, in that if you return a Tuple of the form { :ignore, value } , the value is returned from the call but is not written to the cache. You can use this to abandon writes which began eagerly (for example if a key is actually missing). Examples iex&gt; Cachex . put ( :my_cache , &quot;key&quot; , [ 2 ] ) iex&gt; Cachex . get_and_update ( :my_cache , &quot;key&quot; , &amp; ( [ 1 | &amp;1 ] ) ) { :commit , [ 1 , 2 ] } iex&gt; Cachex . get_and_update ( :my_cache , &quot;missing_key&quot; , fn ...&gt; ( nil ) -&gt; { :ignore , nil } ...&gt; ( val ) -&gt; { :commit , [ &quot;value&quot; | val ] } ...&gt; end ) { :ignore , nil }","ref":"Cachex.html#get_and_update/4","title":"Cachex.get_and_update/4","type":"function"},{"doc":"Imports an export set into a cache. This provides a raw import of a previously exported cache via the use of the export/2 command. ## Examples iex&gt; Cachex . put ( :my_cache , &quot;key&quot; , &quot;value&quot; ) iex&gt; Cachex . import ( :my_cache , [ { :entry , &quot;key&quot; , 1538714590095 , nil , &quot;value&quot; } ] ) { :ok , true }","ref":"Cachex.html#import/3","title":"Cachex.import/3","type":"function"},{"doc":"Increments an entry in the cache. This will overwrite any value that was previously set against the provided key. Options :initial An initial value to set the key to if it does not exist. This will take place before the increment call. Defaults to 0. Examples iex&gt; Cachex . put ( :my_cache , &quot;my_key&quot; , 10 ) iex&gt; Cachex . incr ( :my_cache , &quot;my_key&quot; ) { :ok , 11 } iex&gt; Cachex . put ( :my_cache , &quot;my_new_key&quot; , 10 ) iex&gt; Cachex . incr ( :my_cache , &quot;my_new_key&quot; , 5 ) { :ok , 15 } iex&gt; Cachex . incr ( :my_cache , &quot;missing_key&quot; , 5 , initial : 2 ) { :ok , 7 }","ref":"Cachex.html#incr/4","title":"Cachex.incr/4","type":"function"},{"doc":"Inspects various aspects of a cache. These operations should be regarded as debug tools, and should really only happen outside of production code (unless absolutely) necessary. Accepted options are only provided for convenience and should not be heavily relied upon. They are not part of the public interface (despite being documented) and as such may be removed at any time (however this does not mean that they will be). Please use cautiously. inspect/2 is provided mainly for testing purposes and so performance isn't as much of a concern. It should also be noted that inspect/2 will always operate locally. Options :cache Retrieves the internal cache record for a cache. { :entry, key } Retrieves a raw entry record from inside a cache. { :expired, :count } Retrieves the number of expired entries which currently live in the cache but have not yet been removed by cleanup tasks (either scheduled or lazy). { :expired, :keys } Retrieves the list of expired entry keys which current live in the cache but have not yet been removed by cleanup tasks (either scheduled or lazy). { :janitor, :last } Retrieves metadata about the last execution of the Janitor service for the specified cache. { :memory, :bytes } Retrieves an approximate memory footprint of a cache in bytes. { :memory, :binary } Retrieves an approximate memory footprint of a cache in binary format. { :memory, :words } Retrieve an approximate memory footprint of a cache as a number of machine words. Examples iex&gt; Cachex . inspect ( :my_cache , :cache ) { :ok , { :cache , :my_cache , %{ } , { :expiration , nil , 3000 , true } , { :fallback , nil , nil } , { :hooks , [ ] , [ ] } , { :limit , nil , Cachex.Policy.LRW , 0.1 , [ ] } , false , [ ] } } iex&gt; Cachex . inspect ( :my_cache , { :entry , &quot;my_key&quot; } ) { :ok , { :entry , &quot;my_key&quot; , 1475476615662 , 1 , &quot;my_value&quot; } } iex&gt; Cachex . inspect ( :my_cache , { :expired , :count } ) { :ok , 0 } iex&gt; Cachex . inspect ( :my_cache , { :expired , :keys } ) { :ok , [ ] } iex&gt; Cachex . inspect ( :my_cache , { :janitor , :last } ) { :ok , %{ count : 0 , duration : 57 , started : 1475476530925 } } iex&gt; Cachex . inspect ( :my_cache , { :memory , :binary } ) { :ok , &quot;10.38 KiB&quot; } iex&gt; Cachex . inspect ( :my_cache , { :memory , :bytes } ) { :ok , 10624 } iex&gt; Cachex . inspect ( :my_cache , { :memory , :words } ) { :ok , 1328 }","ref":"Cachex.html#inspect/3","title":"Cachex.inspect/3","type":"function"},{"doc":"Invokes a custom command against a cache entry. The provided command name must be a valid command which was previously attached to the cache in calls to start_link/2 . Examples iex&gt; import Cachex.Spec iex&gt; iex&gt; Cachex . start_link ( :my_cache , [ ...&gt; commands : [ ...&gt; last : command ( type : :read , execute : &amp; List . last / 1 ) ...&gt; ] ...&gt; ] ) { :ok , _pid } iex&gt; Cachex . put ( :my_cache , &quot;my_list&quot; , [ 1 , 2 , 3 ] ) iex&gt; Cachex . invoke ( :my_cache , :last , &quot;my_list&quot; ) { :ok , 3 }","ref":"Cachex.html#invoke/4","title":"Cachex.invoke/4","type":"function"},{"doc":"Retrieves a list of all entry keys from a cache. The order these keys are returned should be regarded as unordered. Examples iex&gt; Cachex . put ( :my_cache , &quot;key1&quot; , &quot;value1&quot; ) iex&gt; Cachex . put ( :my_cache , &quot;key2&quot; , &quot;value2&quot; ) iex&gt; Cachex . put ( :my_cache , &quot;key3&quot; , &quot;value3&quot; ) iex&gt; Cachex . keys ( :my_cache ) { :ok , [ &quot;key2&quot; , &quot;key1&quot; , &quot;key3&quot; ] } iex&gt; Cachex . clear ( :my_cache ) iex&gt; Cachex . keys ( :my_cache ) { :ok , [ ] }","ref":"Cachex.html#keys/2","title":"Cachex.keys/2","type":"function"},{"doc":"Deserializes a cache from a location on a filesystem. This operation will read the current state of a cache from a provided location on a filesystem. This function will only understand files which have previously been created using dump/3 . It is the responsibility of the user to ensure that the location is able to be read from, not the responsibility of Cachex. Options :trusted Allow for loading from trusted or untrusted sources; trusted sources can load atoms into the table, whereas untrusted sources cannot. Defaults to true . Examples iex&gt; Cachex . put ( :my_cache , &quot;my_key&quot; , 10 ) iex&gt; Cachex . dump ( :my_cache , &quot;/tmp/my_backup&quot; ) { :ok , true } iex&gt; Cachex . size ( :my_cache ) { :ok , 1 } iex&gt; Cachex . clear ( :my_cache ) iex&gt; Cachex . size ( :my_cache ) { :ok , 0 } iex&gt; Cachex . load ( :my_cache , &quot;/tmp/my_backup&quot; ) { :ok , true } iex&gt; Cachex . size ( :my_cache ) { :ok , 1 }","ref":"Cachex.html#load/3","title":"Cachex.load/3","type":"function"},{"doc":"Removes an expiration time from an entry in a cache. Examples iex&gt; Cachex . put ( :my_cache , &quot;key&quot; , &quot;value&quot; , ttl : 1000 ) iex&gt; Cachex . persist ( :my_cache , &quot;key&quot; ) { :ok , true } iex&gt; Cachex . persist ( :my_cache , &quot;missing_key&quot; ) { :ok , false }","ref":"Cachex.html#persist/3","title":"Cachex.persist/3","type":"function"},{"doc":"Triggers a cleanup of all expired entries in a cache. This can be used to implement custom eviction policies rather than relying on the internal Janitor service. Take care when using this method though; calling purge/2 manually will result in a purge firing inside the calling process. Examples iex&gt; Cachex . purge ( :my_cache ) { :ok , 15 }","ref":"Cachex.html#purge/2","title":"Cachex.purge/2","type":"function"},{"doc":"Places an entry in a cache. This will overwrite any value that was previously set against the provided key, and overwrite any TTLs which were already set. Options :ttl An expiration time to set for the provided key (time-to-live), overriding any default expirations set on a cache. This value should be in milliseconds. Examples iex&gt; Cachex . put ( :my_cache , &quot;key&quot; , &quot;value&quot; ) { :ok , true } iex&gt; Cachex . put ( :my_cache , &quot;key&quot; , &quot;value&quot; , ttl : :timer . seconds ( 5 ) ) iex&gt; Cachex . ttl ( :my_cache , &quot;key&quot; ) { :ok , 5000 }","ref":"Cachex.html#put/4","title":"Cachex.put/4","type":"function"},{"doc":"Places a batch of entries in a cache. This operates in the same way as put/4 , except that multiple keys can be inserted in a single atomic batch. This is a performance gain over writing keys using multiple calls to put/4 , however it's a performance penalty when writing a single key pair due to some batching overhead. Options :ttl An expiration time to set for the provided keys (time-to-live), overriding any default expirations set on a cache. This value should be in milliseconds. Examples iex&gt; Cachex . put_many ( :my_cache , [ { &quot;key&quot; , &quot;value&quot; } ] ) { :ok , true } iex&gt; Cachex . put_many ( :my_cache , [ { &quot;key&quot; , &quot;value&quot; } ] , ttl : :timer . seconds ( 5 ) ) iex&gt; Cachex . ttl ( :my_cache , &quot;key&quot; ) { :ok , 5000 }","ref":"Cachex.html#put_many/3","title":"Cachex.put_many/3","type":"function"},{"doc":"Refreshes an expiration for an entry in a cache. Refreshing an expiration will reset the existing expiration with an offset from the current time - i.e. if you set an expiration of 5 minutes and wait 3 minutes before refreshing, the entry will expire 8 minutes after the initial insertion. Examples iex&gt; Cachex . put ( :my_cache , &quot;my_key&quot; , &quot;my_value&quot; , ttl : :timer . seconds ( 5 ) ) iex&gt; Process . sleep ( 4 ) iex&gt; Cachex . ttl ( :my_cache , &quot;my_key&quot; ) { :ok , 1000 } iex&gt; Cachex . refresh ( :my_cache , &quot;my_key&quot; ) iex&gt; Cachex . ttl ( :my_cache , &quot;my_key&quot; ) { :ok , 5000 } iex&gt; Cachex . refresh ( :my_cache , &quot;missing_key&quot; ) { :ok , false }","ref":"Cachex.html#refresh/3","title":"Cachex.refresh/3","type":"function"},{"doc":"Resets a cache by clearing the keyspace and restarting any hooks. Options :hooks A whitelist of hooks to reset on the cache instance (call the initialization phase of a hook again). This will default to resetting all hooks associated with a cache, which is usually the desired behaviour. :only A whitelist of components to reset, which can currently contain either the :cache or :hooks tag to determine what to reset. This will default to [ :cache, :hooks ] . Examples iex&gt; Cachex . put ( :my_cache , &quot;my_key&quot; , &quot;my_value&quot; ) iex&gt; Cachex . reset ( :my_cache ) iex&gt; Cachex . size ( :my_cache ) { :ok , 0 } iex&gt; Cachex . reset ( :my_cache , [ only : :hooks ] ) { :ok , true } iex&gt; Cachex . reset ( :my_cache , [ only : :hooks , hooks : [ MyHook ] ] ) { :ok , true } iex&gt; Cachex . reset ( :my_cache , [ only : :cache ] ) { :ok , true }","ref":"Cachex.html#reset/2","title":"Cachex.reset/2","type":"function"},{"doc":"Deprecated implementation delegate of put/4 .","ref":"Cachex.html#set/4","title":"Cachex.set/4","type":"function"},{"doc":"Deprecated implementation delegate of put_many/3 .","ref":"Cachex.html#set_many/3","title":"Cachex.set_many/3","type":"function"},{"doc":"Retrieves the total size of a cache. This does not take into account the expiration time of any entries inside the cache. Due to this, this call is O(1) rather than the more expensive O(n) algorithm used by count/3 . Which you use depends on exactly what you want the returned number to represent. Examples iex&gt; Cachex . put ( :my_cache , &quot;key1&quot; , &quot;value1&quot; ) iex&gt; Cachex . put ( :my_cache , &quot;key2&quot; , &quot;value2&quot; ) iex&gt; Cachex . put ( :my_cache , &quot;key3&quot; , &quot;value3&quot; ) iex&gt; Cachex . size ( :my_cache ) { :ok , 3 }","ref":"Cachex.html#size/2","title":"Cachex.size/2","type":"function"},{"doc":"Creates a new Cachex cache service tree. This will not link the cache to the current process, so if your process dies the cache will continue to live. If you don't want this behaviour, please use the provided start_link/2 . This function is otherwise identical to start_link/2 so please see that documentation for further information and configuration.","ref":"Cachex.html#start/2","title":"Cachex.start/2","type":"function"},{"doc":"Creates a new Cachex cache service tree, linked to the current process. This will link the cache to the current process, so if your process dies the cache will also die. If you don't want this behaviour, please use start/2 . The first argument should be a unique atom, used as the name of the cache service for future calls through to Cachex. For all options requiring a record argument, please import Cachex.Spec in advance. Options :commands This option allows you to attach a set of custom commands to a cache in order to provide shorthand execution. A cache command must be constructed using the :command record provided by Cachex.Spec . A cache command will adhere to these basic rules: If you define a :read command, the return value of your command will be passed through as the result of your call to invoke/4 . If you define a :write command, your command must return a two-element Tuple. The first element represents the value being returned from your invoke/4 call, and the second represents the value to write back into the cache (as an update). If your command does not fit this, errors will happen (intentionally). Commands are set on a per-cache basis, but can be reused across caches. They're set only on cache startup and cannot be modified after the cache tree is created. iex&gt; import Cachex.Spec ...&gt; ...&gt; Cachex . start_link ( :my_cache , [ ...&gt; commands : [ ...&gt; last : command ( type : :read , execute : &amp; List . last / 1 ) , ...&gt; trim : command ( type : :write , execute : &amp; String . trim / 1 ) ...&gt; ] ...&gt; ] ) { :ok , _pid } Either a Keyword or a Map can be provided against the :commands option as we only use Enum to verify them before attaching them internally. Please see the Cachex.Spec.command/1 documentation for further customization options. :compressed This option will specify whether this cache should have enable ETS compression, which is likely to reduce memory overhead. Please note that there is a potential for this option to slow your cache due to compression overhead, so benchmark as appropriate when using this option. This option defaults to false . iex&gt; Cachex . start_link ( :my_cache , [ compressed : true ] ) { :ok , _pid } :expiration The expiration option provides the ability to customize record expiration at a global cache level. The value provided here must be a valid :expiration record provided by Cachex.Spec . iex&gt; import Cachex.Spec ...&gt; ...&gt; Cachex . start_link ( :my_cache , [ ...&gt; expiration : expiration ( ...&gt; # default record expiration ...&gt; default : :timer . seconds ( 60 ) , ...&gt; ...&gt; # how often cleanup should occur ...&gt; interval : :timer . seconds ( 30 ) , ...&gt; ...&gt; # whether to enable lazy checking ...&gt; lazy : true ...&gt; ) ...&gt; ] ) { :ok , _pid } Please see the Cachex.Spec.expiration/1 documentation for further customization options. :fallback The fallback option allows global settings related to the fetch/4 command on a cache. The value provided here can either be a valid :fallback record provided by Cachex.Spec , or a single function (which is turned into a record internally). iex&gt; import Cachex.Spec ...&gt; ...&gt; Cachex . start_link ( :my_cache , [ ...&gt; fallback : fallback ( ...&gt; # default func to use with fetch/4 ...&gt; default : &amp; String . reverse / 1 , ...&gt; ...&gt; # anything to pass to fallbacks ...&gt; provide : { } ...&gt; ) ...&gt; ] ) { :ok , _pid } The :default function provided will be used if fetch/2 is called, rather than explicitly passing one at call time. The :provide function contains state which can be passed to a fallback function if the arity is 2 rather than 1. Please see the documentation for fetch/4 , and the Cachex.Spec.fallback/1 documentation for further information. :hooks The :hooks option allow the user to attach a list of notification hooks to enable listening on cache actions (either before or after they happen). These hooks should be valid :hook records provided by Cachex.Spec . Example hook implementations can be found in Cachex.Stats and Cachex.Policy.LRW . iex&gt; import Cachex.Spec ...&gt; ...&gt; Cachex . start_link ( :my_cache , [ ...&gt; hooks : [ ...&gt; hook ( module : MyHook , name : :my_hook , state : { } ) ...&gt; ] ...&gt; ] ) { :ok , _pid } Please see the Cachex.Spec.hook/1 documentation for further customization options. :limit A cache limit provides a maximum size to cap the cache keyspace at. This should be either a positive integer, or a valid :limit record provided by Cachex.Spec . Internally a provided integer will just be coerced to a :limit record with some default values set. iex&gt; import Cachex.Spec ...&gt; ...&gt; Cachex . start_link ( :my_cache , [ ...&gt; # simple limit ...&gt; limit : 500 , ...&gt; ...&gt; # complex limit ...&gt; limit : limit ( ...&gt; size : 500 , ...&gt; policy : Cachex.Policy.LRW , ...&gt; reclaim : 0.5 , ...&gt; options : [ ] ...&gt; ) ...&gt; ] ) { :ok , _pid } Please see the Cachex.Spec.limit/1 documentation for further customization options. :nodes A list of nodes this cache will live on, to provide distributed behaviour across physical nodes. This should be a list of node names, in the long form. iex&gt; Cachex . start_link ( :my_cache , [ ...&gt; nodes : [ ...&gt; :foo@localhost , ...&gt; :bar@localhost ...&gt; ] ...&gt; ] ) { :ok , _pid } :stats This option can be used to toggle statistics gathering for a cache. This is a shorthand option to avoid attaching the Cachex.Stats hook manually. Statistics gathering has very minor overhead due to being implemented as a hook, Stats can be retrieve from a running cache by using Cachex.stats/2 . iex&gt; Cachex . start_link ( :my_cache , [ stats : true ] ) { :ok , _pid } :transactional This option will specify whether this cache should have transactions and row locking enabled from cache startup. Please note that even if this is false, it will be enabled the moment a transaction is executed. It's recommended to leave this as default as it will handle most use cases in the most performant way possible. iex&gt; Cachex . start_link ( :my_cache , [ transactions : true ] ) { :ok , _pid }","ref":"Cachex.html#start_link/1","title":"Cachex.start_link/1","type":"function"},{"doc":"Retrieves statistics about a cache. This will only provide statistics if the :stats option was provided on cache startup in start_link/2 . Options :for Allows customization of exactly which statistics to retrieve. Examples iex&gt; Cachex . stats ( :my_cache ) { :ok , %{ meta : %{ creation_date : 1518984857331 } } } iex&gt; Cachex . stats ( :cache_with_no_stats ) { :error , :stats_disabled }","ref":"Cachex.html#stats/2","title":"Cachex.stats/2","type":"function"},{"doc":"Creates a Stream of entries in a cache. This will stream all entries matching the match specification provided as the second argument. If none is provided, it will default to all entries which are yet to expire (in no particular order). Consider using Cachex.Query to generate match specifications used when querying the contents of a cache table. Options :batch_size Allows customization of the internal batching when paginating the QLC cursor coming back from ETS. It's unlikely this will ever need changing. Examples iex&gt; Cachex . put ( :my_cache , &quot;a&quot; , 1 ) iex&gt; Cachex . put ( :my_cache , &quot;b&quot; , 2 ) iex&gt; Cachex . put ( :my_cache , &quot;c&quot; , 3 ) { :ok , true } iex&gt; :my_cache |&gt; Cachex . stream! |&gt; Enum . to_list [ { :entry , &quot;b&quot; , 1519015801794 , nil , 2 } , { :entry , &quot;c&quot; , 1519015805679 , nil , 3 } , { :entry , &quot;a&quot; , 1519015794445 , nil , 1 } ] iex&gt; query = Cachex.Query . create ( true , :key ) iex&gt; :my_cache |&gt; Cachex . stream! ( query ) |&gt; Enum . to_list [ &quot;b&quot; , &quot;c&quot; , &quot;a&quot; ] iex&gt; query = Cachex.Query . create ( true , :value ) iex&gt; :my_cache |&gt; Cachex . stream! ( query ) |&gt; Enum . to_list [ 2 , 3 , 1 ] iex&gt; query = Cachex.Query . create ( true , { :key , :value } ) iex&gt; :my_cache |&gt; Cachex . stream! ( query ) |&gt; Enum . to_list [ { &quot;b&quot; , 2 } , { &quot;c&quot; , 3 } , { &quot;a&quot; , 1 } ]","ref":"Cachex.html#stream/3","title":"Cachex.stream/3","type":"function"},{"doc":"Takes an entry from a cache. This is conceptually equivalent to running get/3 followed by an atomic del/3 call. Examples iex&gt; Cachex . put ( :my_cache , &quot;key&quot; , &quot;value&quot; ) iex&gt; Cachex . take ( :my_cache , &quot;key&quot; ) { :ok , &quot;value&quot; } iex&gt; Cachex . get ( :my_cache , &quot;key&quot; ) { :ok , nil } iex&gt; Cachex . take ( :my_cache , &quot;missing_key&quot; ) { :ok , nil }","ref":"Cachex.html#take/3","title":"Cachex.take/3","type":"function"},{"doc":"Updates the last write time on a cache entry. This is very similar to refresh/3 except that the expiration time is maintained inside the record (using a calculated offset).","ref":"Cachex.html#touch/3","title":"Cachex.touch/3","type":"function"},{"doc":"Executes multiple functions in the context of a transaction. This will operate in the same way as execute/3 , except that writes to the specified keys will be blocked on the execution of this transaction. The keys parameter should be a list of keys you wish to lock whilst your transaction is executed. Any keys not in this list can still be written even during your transaction. Examples iex&gt; Cachex . put ( :my_cache , &quot;key1&quot; , &quot;value1&quot; ) iex&gt; Cachex . put ( :my_cache , &quot;key2&quot; , &quot;value2&quot; ) iex&gt; Cachex . transaction ( :my_cache , [ &quot;key1&quot; , &quot;key2&quot; ] , fn ( worker ) -&gt; ...&gt; val1 = Cachex . get ( worker , &quot;key1&quot; ) ...&gt; val2 = Cachex . get ( worker , &quot;key2&quot; ) ...&gt; [ val1 , val2 ] ...&gt; end ) { :ok , [ &quot;value1&quot; , &quot;value2&quot; ] }","ref":"Cachex.html#transaction/4","title":"Cachex.transaction/4","type":"function"},{"doc":"Retrieves the expiration for an entry in a cache. This is a millisecond value (if set) representing the time a cache entry has left to live in a cache. It can return nil if the entry does not have a set expiration. Examples iex&gt; Cachex . ttl ( :my_cache , &quot;my_key&quot; ) { :ok , 13985 } iex&gt; Cachex . ttl ( :my_cache , &quot;my_key_with_no_ttl&quot; ) { :ok , nil } iex&gt; Cachex . ttl ( :my_cache , &quot;missing_key&quot; ) { :ok , nil }","ref":"Cachex.html#ttl/3","title":"Cachex.ttl/3","type":"function"},{"doc":"Updates an entry in a cache. Unlike get_and_update/4 , this does a blind overwrite of a value. This operation can be seen as an internal mutation, meaning that any previously set expiration time is kept as-is. Examples iex&gt; Cachex . put ( :my_cache , &quot;key&quot; , &quot;value&quot; ) iex&gt; Cachex . get ( :my_cache , &quot;key&quot; ) { :ok , &quot;value&quot; } iex&gt; Cachex . update ( :my_cache , &quot;key&quot; , &quot;new_value&quot; ) iex&gt; Cachex . get ( :my_cache , &quot;key&quot; ) { :ok , &quot;new_value&quot; } iex&gt; Cachex . update ( :my_cache , &quot;missing_key&quot; , &quot;new_value&quot; ) { :ok , false }","ref":"Cachex.html#update/4","title":"Cachex.update/4","type":"function"},{"doc":"","ref":"Cachex.html#t:cache/0","title":"Cachex.cache/0","type":"type"},{"doc":"","ref":"Cachex.html#t:status/0","title":"Cachex.status/0","type":"type"},{"doc":"Parent actions module for cache interactions. This module contains foundation actions required to implement cache actions, such as typical CRUD style operations on cache entries. It also provides the defaction/2 macro which enables command definition which injects notifications for cache hooks.","ref":"Cachex.Actions.html","title":"Cachex.Actions","type":"module"},{"doc":"Normalizes a value into a Courier-friendly tagged Tuple. If the value is tagged with :commit , :ignore or :error , it will be left alone; otherwise it will be wrapped and treated as a :commit Tuple.","ref":"Cachex.Actions.html#normalize_commit/1","title":"Cachex.Actions.normalize_commit/1","type":"macro"},{"doc":"Retrieves an entry from a cache. If the entry does not exist, a nil value will be returned. Likewise if the entry has expired, we lazily remove it (if enabled) and return a nil value. This will return an instance of an entry record as defined in the main Cachex.Spec module, rather than just the raw value.","ref":"Cachex.Actions.html#read/2","title":"Cachex.Actions.read/2","type":"function"},{"doc":"Updates a collection of fields inside a cache entry. This is done in a single call due to the use of :ets.update_element/3 which allows multiple changes in a group. This will return a boolean to represent whether the update was successful or not. Note that updates are atomic; either all updates will take place, or none will.","ref":"Cachex.Actions.html#update/3","title":"Cachex.Actions.update/3","type":"function"},{"doc":"Writes a new entry into a cache.","ref":"Cachex.Actions.html#write/2","title":"Cachex.Actions.write/2","type":"function"},{"doc":"Returns the operation used for a write based on a prior value.","ref":"Cachex.Actions.html#write_op/1","title":"Cachex.Actions.write_op/1","type":"function"},{"doc":"Application callback to start any global services. This will start all needed services for Cachex using the Cachex.Services module, rather than hardcoding any logic into this binding module.","ref":"Cachex.Application.html","title":"Cachex.Application","type":"module"},{"doc":"Starts the global services tree for Cachex.","ref":"Cachex.Application.html#start/2","title":"Cachex.Application.start/2","type":"function"},{"doc":"Module dedicated to basic filesystem iteractions. This module contains the required interactions with a filesystem for serializing terms directly to a given file path. This is mainly used by the backup/restore feature of a cache in order to provide easy export functionality. The behaviours in here are general enough that they can be used for various use cases rather than just cache serialization, and compression can also be controlled.","ref":"Cachex.Disk.html","title":"Cachex.Disk","type":"module"},{"doc":"Reads a file from a filesystem using the Erlang Term Format. If there's an error reading the file, or the file is invalid ETF, an error will be returned. Otherwise a Tuple containing the terms will be returned. As we can't be certain what we're reading from the file, we make sure to load it safely to avoid malicious content (although the chance of that is slim).","ref":"Cachex.Disk.html#read/2","title":"Cachex.Disk.read/2","type":"function"},{"doc":"Writes a value to a filesystem using the Erlang Term Format. The compression can be controlled using the :compression option in order to reduce the size of the output. By default this value will be set to level 1 compression. If set to 0, compression will be disabled but be aware storage will increase dramatically.","ref":"Cachex.Disk.html#write/3","title":"Cachex.Disk.write/3","type":"function"},{"doc":"Module containing all error definitions used in the codebase. All error messages (both shorthand and long form) can be found in this module, including the ability to convert from the short form to the long form using the long_form/1 function. This module is provided to allow functions to return short errors, using the easy syntax of error(:short_name) to generate a tuple of { :error, :short_name } but also to allow them to be converted to a readable form as needed, rather than bloating blocks with potentially large error messages.","ref":"Cachex.Errors.html","title":"Cachex.Errors","type":"module"},{"doc":"Generates a tagged :error Tuple at compile time. The provided error key must be contained in the list of known identifiers returned be known/0 , otherwise this call will fail.","ref":"Cachex.Errors.html#error/1","title":"Cachex.Errors.error/1","type":"macro"},{"doc":"Returns the list of known error keys.","ref":"Cachex.Errors.html#known/0","title":"Cachex.Errors.known/0","type":"function"},{"doc":"Converts an error identifier to it's longer form. Error identifiers should be atoms and should be contained in the list of errors returned by known/0 . The return type from this function will always be a binary.","ref":"Cachex.Errors.html#long_form/1","title":"Cachex.Errors.long_form/1","type":"function"},{"doc":"Module controlling hook behaviour definitions. This module defines the hook implementations for Cachex, allowing the user to add hooks into the command execution. This means that users can build plugin style listeners in order to do things like logging. Hooks can be registered to execute either before or after the Cachex command, and can be blocking as needed.","ref":"Cachex.Hook.html","title":"Cachex.Hook","type":"behaviour"},{"doc":"Returns the actions this hook is expected to listen on. This will default to the atom :all , which signals that all actions should be reported to the hook. If not this atom, an enumerable of atoms should be returned.","ref":"Cachex.Hook.html#c:actions/0","title":"Cachex.Hook.actions/0","type":"callback"},{"doc":"Returns whether this hook is asynchronous or not.","ref":"Cachex.Hook.html#c:async?/0","title":"Cachex.Hook.async?/0","type":"callback"},{"doc":"Handles a cache notification. The first argument is the action being taken along with arguments, with the second argument being the results of the action (this can be nil for hooks) which fire before the action is executed.","ref":"Cachex.Hook.html#c:handle_notify/3","title":"Cachex.Hook.handle_notify/3","type":"callback"},{"doc":"Handles a provisioning call. The provided argument will be a Tuple dictating the type of value being provisioned along with the value itself. This can be used to listen on states required for hook executions (such as cache records).","ref":"Cachex.Hook.html#c:handle_provision/2","title":"Cachex.Hook.handle_provision/2","type":"callback"},{"doc":"Returns an enumerable of provisions this hook requires. The current provisions available to a hook are: cache - a cache instance used to make cache calls from inside a hook with zero overhead. This should always return an enumerable of atoms; in the case of no required provisions an empty enumerable should be returned.","ref":"Cachex.Hook.html#c:provisions/0","title":"Cachex.Hook.provisions/0","type":"callback"},{"doc":"Returns the timeout for all calls to this hook. This will be applied to hooks regardless of whether they're synchronous or not; a behaviour change which shipped in v3.0 initially.","ref":"Cachex.Hook.html#c:timeout/0","title":"Cachex.Hook.timeout/0","type":"callback"},{"doc":"Returns the type of this hook. This should return :post to fire after a cache action has occurred, and return :pre if it should fire before the action occurs.","ref":"Cachex.Hook.html#c:type/0","title":"Cachex.Hook.type/0","type":"callback"},{"doc":"Binding module to parse options into a cache record. This interim module is required to normalize the options passed to a cache at startup into a well formed record instance, allowing the rest of the codebase to make assumptions about what types of data are being dealt with.","ref":"Cachex.Options.html","title":"Cachex.Options","type":"module"},{"doc":"Retrieves a conditional option from a Keyword List. If the value satisfies the condition provided, it will be returned. Otherwise the default value provided is returned instead. Used for basic validations.","ref":"Cachex.Options.html#get/4","title":"Cachex.Options.get/4","type":"function"},{"doc":"Parses a list of cache options into a cache record. This will validate any options and error on anything we don't understand. The advantage of binding into a cache instance is that we can blindly use it in other areas of the library without needing to validate. As such, this code can easily become a little messy - but that's ok!","ref":"Cachex.Options.html#parse/2","title":"Cachex.Options.parse/2","type":"function"},{"doc":"Transforms and returns an option inside a Keyword List.","ref":"Cachex.Options.html#transform/3","title":"Cachex.Options.transform/3","type":"function"},{"doc":"Module controlling policy behaviour definitions. This module purely exposes the behaviour and convenience macros for creating a custom policy. It's used internally be Cachex.Policy.LRW and provides very little more than an interface to adhere to.","ref":"Cachex.Policy.html","title":"Cachex.Policy","type":"behaviour"},{"doc":"Returns any hook definitions required for this policy.","ref":"Cachex.Policy.html#c:hooks/1","title":"Cachex.Policy.hooks/1","type":"callback"},{"doc":"Least recently written eviction policies for Cachex. This module provides general utilities for implementing an eviction policy for Cachex which will evict the least-recently written entries from the cache. This is determined by the touched time inside each cache record, which means that we don't have to store any additional tables to keep track of access time. There are several options recognised by this policy which can be passed inside the limit structure when configuring your cache at startup: :batch_size The batch size to use when paginating the cache to evict records. This defaults to 100, which is typically going to be fine for most cases, but this option is exposed in case there is need to customize it. :frequency When this policy operates in scheduled mode, this option controls the frequency with which bounds will be checked. This is specified in milliseconds, and will default to once per second (1000). Feel free to tune this based on how strictly you wish to enforce your cache limits. :immediate Sets this policy to enforce bounds reactively. If this option is set to true , bounds will be checked immediately when a write is made to the cache rather than on a timed schedule. This has the result of being much more accurate with the size of a cache, but has higher overhead due to listening on cache writes. Setting this to true will disable the scheduled checks and thus the :frequency option is ignored in this case. While the overall behaviour of this policy should always result in the same outcome, the way it operates internally may change. As such, the internals of this module should not be relied upon and should not be considered part of the public API.","ref":"Cachex.Policy.LRW.html","title":"Cachex.Policy.LRW","type":"module"},{"doc":"Configures hooks required to back this policy.","ref":"Cachex.Policy.LRW.html#hooks/1","title":"Cachex.Policy.LRW.hooks/1","type":"function"},{"doc":"Evented least recently written eviction policy for Cachex. This module implements an evented LRW eviction policy for Cachex, using a hook to listen for new key additions to a cache and enforcing bounds in a reactive way. This policy enforces cache bounds and limits far more accurately than other scheduled implementations, but comes at a higher memory cost (due to the message passing between hooks).","ref":"Cachex.Policy.LRW.Evented.html","title":"Cachex.Policy.LRW.Evented","type":"module"},{"doc":"Returns the actions this policy should listen on.","ref":"Cachex.Policy.LRW.Evented.html#actions/0","title":"Cachex.Policy.LRW.Evented.actions/0","type":"function"},{"doc":"Returns the provisions this policy requires.","ref":"Cachex.Policy.LRW.Evented.html#provisions/0","title":"Cachex.Policy.LRW.Evented.provisions/0","type":"function"},{"doc":"Scheduled least recently written eviction policy for Cachex. This module implements a scheduled LRW eviction policy for Cachex, using a basic timer to trigger bound enforcement in a repeatable way. This has the same bound accuracy as Cachex.Policy.LRW.Evented , but has potential for some delay. The main advantage of this implementation is a far lower memory cost due to not using hook messages.","ref":"Cachex.Policy.LRW.Scheduled.html","title":"Cachex.Policy.LRW.Scheduled","type":"module"},{"doc":"Returns the actions this policy should listen on.","ref":"Cachex.Policy.LRW.Scheduled.html#actions/0","title":"Cachex.Policy.LRW.Scheduled.actions/0","type":"function"},{"doc":"Returns the provisions this policy requires.","ref":"Cachex.Policy.LRW.Scheduled.html#provisions/0","title":"Cachex.Policy.LRW.Scheduled.provisions/0","type":"function"},{"doc":"Utility module based around creation of cache queries. Queries are essentially just some minor convenience wrappers around the internal match specification used by ETS. This module is exposed to make it easier to query a cache (via Cachex.stream/3 ) without having to take care of filtering for expirations by hand. Note that there is almost no validation in here, so test thoroughly and store compile-time versions of your queries when possible (as performance is not taken into account inside this module; it can be slow to generate).","ref":"Cachex.Query.html","title":"Cachex.Query","type":"module"},{"doc":"Creates an expiration-aware query.","ref":"Cachex.Query.html#create/2","title":"Cachex.Query.create/2","type":"function"},{"doc":"Creates a query to retrieve all expired records.","ref":"Cachex.Query.html#expired/1","title":"Cachex.Query.expired/1","type":"function"},{"doc":"Creates a match condition for expired records.","ref":"Cachex.Query.html#expired_clause/0","title":"Cachex.Query.expired_clause/0","type":"function"},{"doc":"Creates a raw query, ignoring expiration.","ref":"Cachex.Query.html#raw/2","title":"Cachex.Query.raw/2","type":"function"},{"doc":"Creates a query to retrieve all unexpired records.","ref":"Cachex.Query.html#unexpired/1","title":"Cachex.Query.unexpired/1","type":"function"},{"doc":"Creates a match condition for unexpired records.","ref":"Cachex.Query.html#unexpired_clause/0","title":"Cachex.Query.unexpired_clause/0","type":"function"},{"doc":"Routing module to dispatch Cachex actions to their execution environment. This module acts as the single source of dispatch within Cachex. In prior versions the backing actions were called directly from the main interface and were wrapped in macros, which was difficult to maintain and also quite noisy. Now that all execution flows via the router, this is no longer an issue and it also serves as a gateway to distribution in the future.","ref":"Cachex.Router.html","title":"Cachex.Router","type":"module"},{"doc":"Dispatches a call to an appropriate execution environment. This acts as a macro just to avoid the overhead of slicing up module names are runtime, when they can be guaranteed at compile time much more easily.","ref":"Cachex.Router.html#call/2","title":"Cachex.Router.call/2","type":"macro"},{"doc":"Executes a previously dispatched action. This macro should not be called externally; the only reason it remains public is due to the code injected by the dispatch/2 macro.","ref":"Cachex.Router.html#execute/3","title":"Cachex.Router.execute/3","type":"macro"},{"doc":"Service specification provider for Cachex caches. Services can either exist for the global Cachex application or on a cache level. This module provides access to both in an attempt to group all logic into one place to make it easier to see exactly what exists against a cache and what doesn't.","ref":"Cachex.Services.html","title":"Cachex.Services","type":"module"},{"doc":"Returns a list of workers of supervisors for the global app. This will typically only be called once at startup, but it's separated out in order to make it easier to find when comparing supervisors. At the time of writing, the order does not matter - but that does not mean this will always be the case, so please be careful when modifying.","ref":"Cachex.Services.html#app_spec/0","title":"Cachex.Services.app_spec/0","type":"function"},{"doc":"Returns a list of workers or supervisors for a cache. This is used to set up the supervision tree on a cache by cache basis, rather than embedding all of this logic into the parent module. Definition order here matters, as there's inter-dependency between each of the child processes (such as the Janitor -&gt; Locksmith).","ref":"Cachex.Services.html#cache_spec/1","title":"Cachex.Services.cache_spec/1","type":"function"},{"doc":"Retrieves the process identifier of the provided service. This will return nil if the service does not exist, or is not running.","ref":"Cachex.Services.html#locate/2","title":"Cachex.Services.locate/2","type":"function"},{"doc":"Returns a list of all running cache services. This is used to view the children of the specified cache, whilst filtering out any services which may not have been started based on the cache options.","ref":"Cachex.Services.html#services/1","title":"Cachex.Services.services/1","type":"function"},{"doc":"Dispatch service to retrieve values from remote calls. The Courier provides the main implementation for fallbacks triggered by calls to the fetch() command. It acts as a synchronized execution for tasks to avoid duplicating calls when loading. The Courier uses a very simple algorithm to determine when to execute a fallback, so there's very little overhead to synchronizing calls through it. As tasks are dispatched via spawned processes, there's very little action actually happening in the service process itself.","ref":"Cachex.Services.Courier.html","title":"Cachex.Services.Courier","type":"module"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Cachex.Services.Courier.html#child_spec/1","title":"Cachex.Services.Courier.child_spec/1","type":"function"},{"doc":"Dispatches the Courier to execute a task. The task provided must be a closure with arity 0, in order to simplify the interfaces internally. This is a blocking remote call which will wait until a result can be loaded.","ref":"Cachex.Services.Courier.html#dispatch/3","title":"Cachex.Services.Courier.dispatch/3","type":"function"},{"doc":"Starts a new Courier process for a cache.","ref":"Cachex.Services.Courier.html#start_link/1","title":"Cachex.Services.Courier.start_link/1","type":"function"},{"doc":"Parent module for all warmer definitions for a cache. The Incubator will control the supervision tree for all warmers that are associated with a cache. This is very minimal supervision, with no linking back except via the Supervisor access functions.","ref":"Cachex.Services.Incubator.html","title":"Cachex.Services.Incubator","type":"module"},{"doc":"Starts a new incubation service for a cache. This will start a Supervisor to hold all warmer processes as defined in the provided cache record. If no warmers are attached in the cache record, this will skip creation to avoid unnecessary processes running.","ref":"Cachex.Services.Incubator.html#start_link/1","title":"Cachex.Services.Incubator.start_link/1","type":"function"},{"doc":"Parent module for all child hook definitions for a cache. This module will control the supervision tree for all hooks that are associated with a cache. The links inside will create a tree to hold all hooks as children, as well as provide utility functions for new notifications being sent to child hooks for a cache.","ref":"Cachex.Services.Informant.html","title":"Cachex.Services.Informant","type":"module"},{"doc":"Broadcasts an action to all pre-hooks in a cache. This will send a nil result, as the result does not yet exist.","ref":"Cachex.Services.Informant.html#broadcast/2","title":"Cachex.Services.Informant.broadcast/2","type":"function"},{"doc":"Broadcasts an action and result to all post-hooks in a cache.","ref":"Cachex.Services.Informant.html#broadcast/3","title":"Cachex.Services.Informant.broadcast/3","type":"function"},{"doc":"Links all hooks in a cache to their running process. This is a required post-step as hooks are started independently and are not named in a deterministic way. It will look up all hooks using the Supervisor children and place them in a modified cache record.","ref":"Cachex.Services.Informant.html#link/1","title":"Cachex.Services.Informant.link/1","type":"function"},{"doc":"Notifies a set of hooks of the passed in data. This is the underlying implementation for broadcast/2 and broadcast/3 , but it's general purpose enough that it's exposed as part of the public API.","ref":"Cachex.Services.Informant.html#notify/3","title":"Cachex.Services.Informant.notify/3","type":"function"},{"doc":"Starts a new Informant service for a cache. This will start a Supervisor to hold all hook processes as defined in the provided cache record. If no hooks are attached in the cache record, this will skip creating an unnecessary Supervisor process.","ref":"Cachex.Services.Informant.html#start_link/1","title":"Cachex.Services.Informant.start_link/1","type":"function"},{"doc":"Expiration service to clean up expired cache records periodically. The Janitor provides the main expiration cleanup for Cachex, providing a very basic scheduler to repeatedly cleanup cache tables for all expired entries. This runs in a separate process to avoid any potential overhead in for a user, but uses existing functions in the API so manual cleanup is possible. It's possible that certain cleanups will result in full table scans, so it should be expected that this can take a while to execute.","ref":"Cachex.Services.Janitor.html","title":"Cachex.Services.Janitor","type":"module"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Cachex.Services.Janitor.html#child_spec/1","title":"Cachex.Services.Janitor.child_spec/1","type":"function"},{"doc":"Pulls an expiration associated with an entry.","ref":"Cachex.Services.Janitor.html#expiration/2","title":"Cachex.Services.Janitor.expiration/2","type":"function"},{"doc":"Determines if a cache entry has expired. This will not cache lazy expiration settings into account.","ref":"Cachex.Services.Janitor.html#expired?/1","title":"Cachex.Services.Janitor.expired?/1","type":"function"},{"doc":"Determines if a cache entry has expired. This will take cache lazy expiration settings into account.","ref":"Cachex.Services.Janitor.html#expired?/2","title":"Cachex.Services.Janitor.expired?/2","type":"function"},{"doc":"Retrieves information about the latest Janitor run for a cache. If the service is disabled on the cache, an error is returned.","ref":"Cachex.Services.Janitor.html#last_run/1","title":"Cachex.Services.Janitor.last_run/1","type":"function"},{"doc":"Starts a new Janitor process for a cache. At this point customization is non-existent, in order to keep the service as simple as possible and avoid the space for error and edge cases.","ref":"Cachex.Services.Janitor.html#start_link/1","title":"Cachex.Services.Janitor.start_link/1","type":"function"},{"doc":"Locking service in charge of table transactions. This module acts as a global lock table against all cache. This is due to the fact that ETS tables are fairly expensive to construct if they're only going to store a few keys. Due to this we have a single global table in charge of locks, and we tag just the key in the table with the name of the cache it's associated with. This keyspace will typically be very small, so there should be almost no impact to operating in this way (except that we only have a single ETS table rather than a potentially large N). It should be noted that the behaviour in this module could easily live as a GenServer if it weren't for the speedup gained when using ETS. When using an ETS table, checking for a lock is typically 0.3-0.5µs/op whereas a call to a server process is roughly 10x this (due to the process interactions).","ref":"Cachex.Services.Locksmith.html","title":"Cachex.Services.Locksmith","type":"module"},{"doc":"Locks a number of keys for a cache. This function can handle multiple keys to lock together atomically. The returned boolean will signal if the lock was successful. A lock can fail if one of the provided keys is already locked.","ref":"Cachex.Services.Locksmith.html#lock/2","title":"Cachex.Services.Locksmith.lock/2","type":"function"},{"doc":"Retrieves a list of locked keys for a cache. This uses some ETS matching voodoo to pull back the locked keys. They won't be returned in any specific order, so don't rely on it.","ref":"Cachex.Services.Locksmith.html#locked/1","title":"Cachex.Services.Locksmith.locked/1","type":"function"},{"doc":"Determines if a key is able to be written to by the current process. For a key to be writeable, it must either have no lock or be locked by the calling process.","ref":"Cachex.Services.Locksmith.html#locked?/2","title":"Cachex.Services.Locksmith.locked?/2","type":"function"},{"doc":"Starts the backing services required by the Locksmith. At this point this will start the backing ETS table required by the locking logic inside the Locksmith. This is started with concurrency enabled and logging disabled to avoid spamming log output. This may become configurable in future, but this table will likelyn never cause issues in the first place (as it only handles very basic operations).","ref":"Cachex.Services.Locksmith.html#start_link/0","title":"Cachex.Services.Locksmith.start_link/0","type":"function"},{"doc":"Flags this process as running in a transaction.","ref":"Cachex.Services.Locksmith.html#start_transaction/0","title":"Cachex.Services.Locksmith.start_transaction/0","type":"function"},{"doc":"Flags this process as not running in a transaction.","ref":"Cachex.Services.Locksmith.html#stop_transaction/0","title":"Cachex.Services.Locksmith.stop_transaction/0","type":"function"},{"doc":"Executes a transaction against a cache table. If the process is already in a transactional context, the provided function will be executed immediately. Otherwise the required keys will be locked until the provided function has finished executing. This is mainly shorthand to avoid having to handle row locking explicitly.","ref":"Cachex.Services.Locksmith.html#transaction/3","title":"Cachex.Services.Locksmith.transaction/3","type":"function"},{"doc":"Determines if the current process is in transactional context.","ref":"Cachex.Services.Locksmith.html#transaction?/0","title":"Cachex.Services.Locksmith.transaction?/0","type":"function"},{"doc":"Unlocks a number of keys for a cache. There's currently no way to batch delete items in ETS beyond a select_delete, so we have to simply iterate over the locks and remove them sequentially. This is a little less desirable, but needs must.","ref":"Cachex.Services.Locksmith.html#unlock/2","title":"Cachex.Services.Locksmith.unlock/2","type":"function"},{"doc":"Performs a write against the given key inside the table. If the key is locked, the write is queued inside the lock server to ensure that we execute consistently. This is a little hard to explain, but if the cache has not had any transactions executed against it we skip the lock check as any of our ETS writes are atomic and so do not require a lock.","ref":"Cachex.Services.Locksmith.html#write/3","title":"Cachex.Services.Locksmith.write/3","type":"function"},{"doc":"Transaction queue backing a cache instance. This has to live outside of the Cachex.Services.Locksmith global process as otherwise caches would then compete with each other for resources which is far from optimal. Each cache will therefore have their own queue process, represented in this module, and will operate using the utilities provided in the main Locksmith service module (rather than using this module directly).","ref":"Cachex.Services.Locksmith.Queue.html","title":"Cachex.Services.Locksmith.Queue","type":"module"},{"doc":"Executes a function in a lock-free context.","ref":"Cachex.Services.Locksmith.Queue.html#execute/2","title":"Cachex.Services.Locksmith.Queue.execute/2","type":"function"},{"doc":"Starts the internal server process backing this queue. This is little more than starting a GenServer process using this module, although it does use the provided cache record to name the new server.","ref":"Cachex.Services.Locksmith.Queue.html#start_link/1","title":"Cachex.Services.Locksmith.Queue.start_link/1","type":"function"},{"doc":"Executes a function in a transactional context.","ref":"Cachex.Services.Locksmith.Queue.html#transaction/3","title":"Cachex.Services.Locksmith.Queue.transaction/3","type":"function"},{"doc":"Service module overseeing the persistence of cache records. This module controls the state of caches being handled by Cachex. This was originally part of an experiment to see if it was viable to remove a process which backed each cache to avoid bottlenecking scenarios and grant the develop finer control over their concurrency. The result was much higher throughput with better flexibility, and so we kept this new design. Cache states are stored in a single ETS table backing this module and all cache calls will be routed through here first to ensure their state is up to date.","ref":"Cachex.Services.Overseer.html","title":"Cachex.Services.Overseer","type":"module"},{"doc":"Ensures a cache from a name or record. Ensuring a cache will map the provided argument to a cache record if available, otherwise a nil value.","ref":"Cachex.Services.Overseer.html#ensure/1","title":"Cachex.Services.Overseer.ensure/1","type":"function"},{"doc":"Determines whether a cache is known by the Overseer.","ref":"Cachex.Services.Overseer.html#known?/1","title":"Cachex.Services.Overseer.known?/1","type":"function"},{"doc":"Registers a cache record against a name.","ref":"Cachex.Services.Overseer.html#register/2","title":"Cachex.Services.Overseer.register/2","type":"function"},{"doc":"Retrieves a cache record, or nil if none exists.","ref":"Cachex.Services.Overseer.html#retrieve/1","title":"Cachex.Services.Overseer.retrieve/1","type":"function"},{"doc":"Creates a new Overseer service tree. This will start a basic Agent for transactional changes, as well as the main ETS table backing this service.","ref":"Cachex.Services.Overseer.html#start_link/0","title":"Cachex.Services.Overseer.start_link/0","type":"function"},{"doc":"Determines whether the Overseer has been started.","ref":"Cachex.Services.Overseer.html#started?/0","title":"Cachex.Services.Overseer.started?/0","type":"function"},{"doc":"Carries out a transaction against the state table.","ref":"Cachex.Services.Overseer.html#transaction/2","title":"Cachex.Services.Overseer.transaction/2","type":"function"},{"doc":"Unregisters a cache record against a name.","ref":"Cachex.Services.Overseer.html#unregister/1","title":"Cachex.Services.Overseer.unregister/1","type":"function"},{"doc":"Updates a cache record against a name. This is atomic and happens inside a transaction to ensure that we don't get out of sync. Hooks are notified of the change, and the new state is returned.","ref":"Cachex.Services.Overseer.html#update/2","title":"Cachex.Services.Overseer.update/2","type":"function"},{"doc":"Specification definitions based around records and utilities. This serves as the &quot;parent&quot; header file for Cachex, where all records and macros are located. It's designed as a single inclusion file which provides everything you might need to implement features in Cachex, and indeed when interacting with Cachex. Most macros in here should be treated as reserved for internal use only, but those based around records can be freely used by consumers of Cachex.","ref":"Cachex.Spec.html","title":"Cachex.Spec","type":"module"},{"doc":"Creates a cache record from the provided values. A cache record is used to represent the internal state of a cache, and is used when executing calls. Most values in here will be other records defined in the main specification, and as such please see their documentation for further info.","ref":"Cachex.Spec.html#cache/1","title":"Cachex.Spec.cache/1","type":"macro"},{"doc":"Updates a cache record from the provided values.","ref":"Cachex.Spec.html#cache/2","title":"Cachex.Spec.cache/2","type":"macro"},{"doc":"Creates a command record from the provided values. A command is a custom action which can be executed against a cache instance. They consist of a type ( :read / :write ) and an execution function. The type determines what form the execution should take. In the case of a :read type, an execution function is a simple (any -&gt; any) form, which will return the returned value directly to the caller. In the case of a :write type, an execution should be (any -&gt; { any, any }) where the value in the left side of the returned Tuple will be returned to the caller, and the right side will be set inside the backing cache table.","ref":"Cachex.Spec.html#command/1","title":"Cachex.Spec.command/1","type":"macro"},{"doc":"Updates a command record from the provided values.","ref":"Cachex.Spec.html#command/2","title":"Cachex.Spec.command/2","type":"macro"},{"doc":"Inserts constant values by a provided key. Constants are meant to only be used internally as they may change without warning, but they are exposed as part of the spec interface all the same. Constant blocks can use other constants in their definitions (as it's all just macros under the hood, and happens at compile time).","ref":"Cachex.Spec.html#const/1","title":"Cachex.Spec.const/1","type":"macro"},{"doc":"Creates an entry record from the provided values. An entry record reprents a single entry in a cache table. Each entry has a key/value, along with a touch time and ttl. These records should never be used outside of the Cachex codebase other than when debugging, as they can change at any time and should be regarded as internal only.","ref":"Cachex.Spec.html#entry/1","title":"Cachex.Spec.entry/1","type":"macro"},{"doc":"Updates an entry record from the provided values.","ref":"Cachex.Spec.html#entry/2","title":"Cachex.Spec.entry/2","type":"macro"},{"doc":"Retrieves the ETS index for an entry field.","ref":"Cachex.Spec.html#entry_idx/1","title":"Cachex.Spec.entry_idx/1","type":"macro"},{"doc":"Generates an ETS modification Tuple for entry field/value pairs. This will convert the entry field name to the ETS index under the hood, and return it inside a Tuple with the provided value.","ref":"Cachex.Spec.html#entry_mod/1","title":"Cachex.Spec.entry_mod/1","type":"macro"},{"doc":"Generates a list of ETS modification Tuples with an updated touch time. This will pass the arguments through and behave exactly as entry_mod/1 except that it will automatically update the :touched field in the entry to the current time.","ref":"Cachex.Spec.html#entry_mod_now/1","title":"Cachex.Spec.entry_mod_now/1","type":"macro"},{"doc":"Creates an entry record with an updated touch time. This delegates through to entry/1 , but ensures that the :touched field is set to the current time as a millisecond timestamp.","ref":"Cachex.Spec.html#entry_now/1","title":"Cachex.Spec.entry_now/1","type":"macro"},{"doc":"Creates an expiration record from the provided values. An expiration record contains properties defining expiration policies for a cache. A default value can be provided which will then be added as a default TTL to all keys which do not have one set explicitly. This must be a positive millisecond integer. The interval being controlled here is the Janitor service schedule; it controls how often the purge runs in the background of your application to remove expired records. This can be disabled completely by setting the value to nil. This is also a millisecond integer. The lazy value determines whether or not records can be lazily removed on read. Since this is an expected behaviour it's enabled by default, but there are cases where you might wish to disable it (such as when consistency isn't that big an issue).","ref":"Cachex.Spec.html#expiration/1","title":"Cachex.Spec.expiration/1","type":"macro"},{"doc":"Updates an expiration record from the provided values.","ref":"Cachex.Spec.html#expiration/2","title":"Cachex.Spec.expiration/2","type":"macro"},{"doc":"Creates a fallback record from the provided values. A fallback can consist of a nillable state to provide to a fallback definition when requested (via a fallback with an arity of 2). If a default action is provided, it should be a function of arity 1 or 2, depending on if it requires the state or not.","ref":"Cachex.Spec.html#fallback/1","title":"Cachex.Spec.fallback/1","type":"macro"},{"doc":"Updates a fallback record from the provided values.","ref":"Cachex.Spec.html#fallback/2","title":"Cachex.Spec.fallback/2","type":"macro"},{"doc":"Creates a hook record from the provided values. Hook records contain the properties needed to start up a hook against a cache instance. There are several components in a hook record: arguments to pass through to the hook init/1 callback. a flag to set whether or not a hook should fire asynchronously. the module name backing the hook, implementing the hook behaviour. options to pass to the hook server instance (allowing for names, etc). provisions to pass through to the hook provisioning callback. a PID reference to a potentially running hook instance (optional). the timeout to wait for a response when firing synchronous hooks. the type of the hook (whether to fire before/after a request). These values are mainly provided by the user, and this record might actually be replaced in future with just a behaviour and a set of macros (as this record is very noisy now).","ref":"Cachex.Spec.html#hook/1","title":"Cachex.Spec.hook/1","type":"macro"},{"doc":"Updates a hook record from the provided values.","ref":"Cachex.Spec.html#hook/2","title":"Cachex.Spec.hook/2","type":"macro"},{"doc":"Creates a hooks collection record from the provided values. Hooks records are just a pre-sorted collection of hook records, grouped by their type so that notifications internally do not have to iterate and group manually.","ref":"Cachex.Spec.html#hooks/1","title":"Cachex.Spec.hooks/1","type":"macro"},{"doc":"Updates a hooks record from the provided values.","ref":"Cachex.Spec.html#hooks/2","title":"Cachex.Spec.hooks/2","type":"macro"},{"doc":"Determines if a value is a negative integer.","ref":"Cachex.Spec.html#is_negative_integer/1","title":"Cachex.Spec.is_negative_integer/1","type":"macro"},{"doc":"Determines if a value is a positive integer.","ref":"Cachex.Spec.html#is_positive_integer/1","title":"Cachex.Spec.is_positive_integer/1","type":"macro"},{"doc":"Creates a limit record from the provided values. A limit record represents size bounds on a cache, and the way size should be reclaimed. A limit should have a valid integer as the maximum cache size, which is used to determine when to cull records. By default, an LRW style policy will be applied to remove old records but this can also be customized using the policy value. The amount of space to reclaim at once can be provided using the reclaim option. You can also specify options to pass through to the policy server, in order to customize policy behaviour.","ref":"Cachex.Spec.html#limit/1","title":"Cachex.Spec.limit/1","type":"macro"},{"doc":"Updates a limit record from the provided values.","ref":"Cachex.Spec.html#limit/2","title":"Cachex.Spec.limit/2","type":"macro"},{"doc":"Generates a named atom for a cache, using the provided service. The list of services is narrowly defined to avoid bloating the atom table as it's not garbage collected. This macro is only used when naming services.","ref":"Cachex.Spec.html#name/2","title":"Cachex.Spec.name/2","type":"macro"},{"doc":"Checks if a nillable value satisfies a provided condition.","ref":"Cachex.Spec.html#nillable?/2","title":"Cachex.Spec.nillable?/2","type":"macro"},{"doc":"Retrieves the current system time in milliseconds.","ref":"Cachex.Spec.html#now/0","title":"Cachex.Spec.now/0","type":"macro"},{"doc":"Generates a service call for a cache. This will generate the service name for the provided cache and call the service with the provided message. The timeout for these service calls is :infinity as they're all able to block the caller.","ref":"Cachex.Spec.html#service_call/3","title":"Cachex.Spec.service_call/3","type":"macro"},{"doc":"Retrieves the currently handled stacktrace.","ref":"Cachex.Spec.html#stack_compat/0","title":"Cachex.Spec.stack_compat/0","type":"macro"},{"doc":"Adds a :via delegation to a Keyword List.","ref":"Cachex.Spec.html#via/2","title":"Cachex.Spec.via/2","type":"macro"},{"doc":"Creates a warmer record from the provided values. A warmer record represents cache warmer processes to be run to populate keys. A warmer should have a valid module provided, which correctly implements the behaviour associated with Cachex.Warmer . A state can also be provided, which will be passed to the execution callback of the provided module (which defaults to nil ).","ref":"Cachex.Spec.html#warmer/1","title":"Cachex.Spec.warmer/1","type":"macro"},{"doc":"Updates a warmer record from the provided values.","ref":"Cachex.Spec.html#warmer/2","title":"Cachex.Spec.warmer/2","type":"macro"},{"doc":"Wraps a value inside a tagged Tuple using the provided tag.","ref":"Cachex.Spec.html#wrap/2","title":"Cachex.Spec.wrap/2","type":"macro"},{"doc":"","ref":"Cachex.Spec.html#t:cache/0","title":"Cachex.Spec.cache/0","type":"type"},{"doc":"","ref":"Cachex.Spec.html#t:command/0","title":"Cachex.Spec.command/0","type":"type"},{"doc":"","ref":"Cachex.Spec.html#t:entry/0","title":"Cachex.Spec.entry/0","type":"type"},{"doc":"","ref":"Cachex.Spec.html#t:expiration/0","title":"Cachex.Spec.expiration/0","type":"type"},{"doc":"","ref":"Cachex.Spec.html#t:fallback/0","title":"Cachex.Spec.fallback/0","type":"type"},{"doc":"","ref":"Cachex.Spec.html#t:hook/0","title":"Cachex.Spec.hook/0","type":"type"},{"doc":"","ref":"Cachex.Spec.html#t:hooks/0","title":"Cachex.Spec.hooks/0","type":"type"},{"doc":"","ref":"Cachex.Spec.html#t:limit/0","title":"Cachex.Spec.limit/0","type":"type"},{"doc":"","ref":"Cachex.Spec.html#t:warmer/0","title":"Cachex.Spec.warmer/0","type":"type"},{"doc":"Validation module for records defined in the specification. This module just exposes runtime validation functions for records defined in the spec; records themselves only determine keys and structure but cannot enforce type (that I know of) without additional runtime validations. This shouldn't be used outside of the library, but it can be if required.","ref":"Cachex.Spec.Validator.html","title":"Cachex.Spec.Validator","type":"module"},{"doc":"Validates a specification record type. This will provide runtime validation of types and values contained inside the specification records. Although records provide key validation, they don't expose much coverage of the provided values. This will delegate each record type to a customized validation function.","ref":"Cachex.Spec.Validator.html#valid?/2","title":"Cachex.Spec.Validator.valid?/2","type":"function"},{"doc":"","ref":"Cachex.Spec.Validator.html#t:record/0","title":"Cachex.Spec.Validator.record/0","type":"type"},{"doc":"Hook module to control the gathering of cache statistics. This implementation of statistics tracking uses a hook to run asynchronously against a cache (so that it doesn't impact those who don't want it). It executes as a post hook and provides a solid example of what a hook can/should look like. This hook has zero knowledge of the cache it belongs to; it keeps track of an internal set of statistics based on the provided messages. This means that it can also be mocked easily using raw server calls to handle_notify/3 .","ref":"Cachex.Stats.html","title":"Cachex.Stats","type":"module"},{"doc":"Determines if stats are enabled for a cache.","ref":"Cachex.Stats.html#enabled?/1","title":"Cachex.Stats.enabled?/1","type":"function"},{"doc":"Locates a stats hook for a cache, if enabled.","ref":"Cachex.Stats.html#locate/1","title":"Cachex.Stats.locate/1","type":"function"},{"doc":"Retrieves the latest statistics for a cache.","ref":"Cachex.Stats.html#retrieve/1","title":"Cachex.Stats.retrieve/1","type":"function"},{"doc":"Module controlling cache warmer behaviour definitions. This module defines the cache warming implementation for Cachex, allowing the user to register warmers against a cache to populate the tables on an interval. Doing this allows for easy pulling against expensive values (such as those from a backing database or remote server), without risking heavy usage. Warmers will block when the cache is started to ensure that the application will not complete booting up, until your cache has been warmed. This guarantees that there will not be any time where the application is available, without the desired data in the cache. Warmers are fired on a schedule, and are exposed via a very simple behaviour of just an interval and a block to execute on the interval. It should be noted that this is a moving interval, and it resets after execution has completed.","ref":"Cachex.Warmer.html","title":"Cachex.Warmer","type":"behaviour"},{"doc":"Executes actions to warm a cache instance on interval. This can either return values to set in the cache, or the atom :ignore to signal that there's nothing to be set at this point in time. Values to be set should be returned as { :ok, pairs } where pairs is a list of { key, value } pairs to place into the cache via Cachex.put_many/3 . If you wish to provide expiration against the keys, you can return a Tuple in the form { :ok, pairs, opts } where opts is a list of options as accepted by the Cachex.put_many/3 function, thus allowing you to expire your warmed data. Unless this is provided, there is no explicit expiration associated with warmed values (as predicting the appropriate expiration is not possible). The argument provided here is the one provided as state to the warmer records at cache configuration time; it will be nil if none was provided.","ref":"Cachex.Warmer.html#c:execute/1","title":"Cachex.Warmer.execute/1","type":"callback"},{"doc":"Returns the interval this warmer will execute on. This must be an integer representing a count of milliseconds to wait before the next execution of the warmer. Anything else will cause either invalidation errors on cache startup, or crashes at runtime.","ref":"Cachex.Warmer.html#c:interval/0","title":"Cachex.Warmer.interval/0","type":"callback"},{"doc":"Minor error implementation for all Cachex-specific errors. This module allows users caton catch Cachex errors in a separate block to other errors/exceptions rather than using stdlib errors. iex&gt; try do ...&gt; Cachex . put! ( :cache , &quot;key&quot; , &quot;value&quot; ) ...&gt; rescue ...&gt; e in Cachex.ExecutionError -&gt; e ...&gt; end The default error message should always be overridden with a long error formas displayed inside Cachex.Errors .","ref":"Cachex.ExecutionError.html","title":"Cachex.ExecutionError","type":"exception"},{"doc":"","ref":"getting-started.html","title":"Getting Started","type":"extras"},{"doc":"To start a cache you can use either start/2 or start_link/2 , and in general you should place it into your Supervision trees for fault tolerance. The first argument is the name of the cache and defines how you will communicate with your cache. Supervisor . start_link ( [ { Cachex , name : :my_cache } ] ) The second and third arguments are both optional and represent cache and server options respectively. Cache options can be set on a cache at startup and cannot be modified. They're defined on a per-cache basis and control the features available to the cache. This table contains a summary of most of the available options, but please look at the module documentation either in GitHub or on Hexdocs for full documentation on what each one can configure. Options Values Description commands map or keyword A collection of custom commands to attach to the cache. expiration expiration() An expiration options record imported from Cachex.Spec. fallback function or fallback() A fallback record imported from Cachex.Spec. hooks list of hook() A list of execution hooks to listen on cache actions. limit a limit() record An integer or Limit struct to define the bounds of this cache. stats boolean Whether to track statistics for this cache or not. transactional boolean Whether to turn on transactions at cache start. warmers list of warmer() A list of cache warmers to enable on the cache.","ref":"getting-started.html#starting-your-cache","title":"Getting Started - Starting Your Cache","type":"extras"},{"doc":"The Cachex interface follows a specific standard to make it easier to predict and more user friendly. All calls should follow the pattern of having the cache argument first, followed by any required arguments and ending with an optional list of options (even if no options are currently used). All calls should result in a value in the format of { status, result } where status is usually :ok or :error (however this differs depending on the call). The result can basically be anything, as there are a number of custom controlled return values available inside Cachex. In the interest of convenience all Cachex actions have an automatically generated &quot;unsafe&quot; equivalent (appended with ! ) which unwraps these result Tuples. This unwrapping assumes that :error status means that the result should be raised, and that any other status should just return the result itself. iex(1)&gt; Cachex . get ( :my_cache , &quot;key&quot; ) { :ok , nil } iex(2)&gt; Cachex . get! ( :my_cache , &quot;key&quot; ) nil iex(3)&gt; Cachex . get ( :missing_cache , &quot;key&quot; ) { :error , :no_cache } iex(4)&gt; Cachex . get! ( :missing_cache , &quot;key&quot; ) ** (Cachex.ExecutionError) Specified cache not running (cachex) lib/cachex.ex:249: Cachex.get!/3 In production code I would typically recommend the safer versions to be explicit but the ! version exists for both convenience and unit test code to make assertions easier.","ref":"getting-started.html#main-interface","title":"Getting Started - Main Interface","type":"extras"},{"doc":"As of v0.9.0 , support for action blocks has been incorporated into Cachex. These blocks provide different ways of executing a batch of actions sequentially inside a modified cache context. This change in context provides differences in behaviour which affect how your cache actions are carried out. Currently, they come in two flavours; execution blocks and transaction blocks. Each block uses a function provided with a state in order to utilise the scope correctly; if you don't pass the state to your cache calls and instead use the cache name, you'll lose out on the advantages of the block.","ref":"action-blocks.html","title":"Action Blocks","type":"extras"},{"doc":"An execution block is a pretty straightforward notion; due to Cachex requiring internal state to carry out a call, we can optimize this by retrieving the state once, executing all actions, then putting the state back. Back in the v1.x line, this was important as the state was stored in a GenServer and so an execution block would be a single GenServer call rather than N calls. Even though this is no longer backed by a GenServer there is still a throughput boost, so consider using these blocks if you're doing several cache calls in a row. To provide an example, consider trying to retrieve two keys from a cache one after another. Below is an example both without and with an execution block: # without is the usual interface val1 = Cachex . get! ( :my_cache , &quot;key1&quot; ) val2 = Cachex . get! ( :my_cache , &quot;key2&quot; ) # this is using an execution block { val1 , val2 } = Cachex . execute! ( :my_cache , fn ( cache ) -&gt; v1 = Cachex . get! ( cache , &quot;key1&quot; ) v2 = Cachex . get! ( cache , &quot;key2&quot; ) { v1 , v2 } end ) The syntax looks a little more complicated to start with, but you'll soon get used to it. It's a small trade off for a potentially large throughput boost (estimated best result would be 1/Nth of the time when N is the number of calls you're making). It's very important to note that even though you're executing a block, other actions from other processes can happen at any time inside your block. To demonstrate this, here's a quick example: # start our execution block Cachex . execute! ( :my_cache , fn ( cache ) -&gt; # set a base value in the cache Cachex . put! ( cache , &quot;key&quot; , &quot;value&quot; ) # we&#39;re paused but other stuff can happen :timer . sleep ( 5000 ) # this may have have been set elsewhere by this point Cachex . get! ( cache , &quot;key&quot; ) end ) As we wait 5 seconds before reading the value back, the value may have been modified or even removed by other processes using the cache (such as TTL cleanup or other places in your application). If you want to guarantee that nothing is modified between your interactions, you should consider a transactional block instead.","ref":"action-blocks.html#execution-blocks","title":"Action Blocks - Execution Blocks","type":"extras"},{"doc":"One of the most useful blocks is the transactional block. These blocks will bind all actions inside into a transaction in order to ensure consistency, meaning that all actions defined in your transaction will execute sequentially with zero interaction from other processes. These blocks are quite similar in definition to execution blocks, except that they require a list of keys to lock throughout execution. Any keys not specified can still be written by other processes due to the optimizations made for locking (or not locking, I guess). # start our execution block Cachex . transaction! ( :my_cache , [ &quot;key&quot; ] , fn ( cache ) -&gt; # set a base value in the cache Cachex . put! ( cache , &quot;key&quot; , &quot;value&quot; ) # we&#39;re paused but other stuff can not happen :timer . sleep ( 5000 ) # this will be guaranteed to return &quot;value&quot; Cachex . get! ( cache , &quot;key&quot; ) end ) Naturally there is an overhead to transactions so use them only when you have to, however they're much more optimized than previous major versions of Cachex (as of v2.x ) in that there should be no visible slowdown to writes against keys which do not have a lock. Transactional blocks are backed by a GenServer so be aware that throughput will line up with (at best) the throughput of GenServer calls.","ref":"action-blocks.html#transaction-blocks","title":"Action Blocks - Transaction Blocks","type":"extras"},{"doc":"Cache limits are restrictions on a cache to ensure that it stays within given bounds. Currently these limits are based around the number of entries inside a cache, but there are plans to add new policies in future (for example basing the limits on memory spaces).","ref":"cache-limits.html","title":"Cache Limits","type":"extras"},{"doc":"Limits are defined at cache startup and cannot be changed at this point in time. You can provide either an integer or a limit record to the :limit option in the Cachex interface. # include records import Cachex.Spec # maximum 500 entries, default eviction, default trim Cachex . start ( :my_cache , [ limit : 500 ] ) # maximum 500 entries, LRW eviction, trim to 250 Cachex . start ( :my_cache , [ limit : limit ( size : 500 , policy : Cachex.Policy.LRW , reclaim : 0.5 ) ] ) A limit record consists (currently) of only 4 fields which dictate a limit and how it should be enforced. This allows the user to customize their eviction without getting too low-level. Below is an example structure, which demonstrates what an integer :limit parameter would unpack to internally. limit ( # the limit provided size : 500 , # the policy to use for eviction policy : Cachex.Policy.LRW , # how much to reclaim on bound expiration reclaim : 0.1 , # options to pass to the policy options : [ ] ) To expound a little on the above, it defines that the cache should aim to store no more than 500 entries (which is user defined). If the cache key space goes above this number, it should evict 50 of the entries in the cache as chosen by the provided :policy . The amount 50 is dictated by the :reclaim option, which is essentially a percentage of the cache to evict on hitting the bounds. This value much match 1 &gt;= value &gt;= 0 in order to be accepted and override the default (due to being a percentage).","ref":"cache-limits.html#configuration","title":"Cache Limits - Configuration","type":"extras"},{"doc":"The policy Cachex.Policy.LRW above is a built-in Cachex eviction policy which removes the oldest values first. This means that we calculate the first N oldest entries, where N is roughly equal to limit * reclaim , and remove them from the cache in order to make room for new entries. It should be noted that &quot;oldest&quot; in this context means &quot;those written or updated longest ago&quot;. This is currently the only policy implemented within Cachex, although it's likely that more will follow (and you can write them yourself too). You should be aware that eviction is not instant - it happens in reaction to events which are additive to the cache and is extremely quick, however if you have a cache limit of 500 keys and you add 500,000 keys, the cleanup does take a few hundred milliseconds to occur (that's a lot to clean). This shouldn't affect most users, but it is something to point out and be aware of. It should be noted that although LRW is the only policy implemented at this time, you can control LRU policies by using Cachex.touch/2 to do a write on a key without affecting the value or TTL. Using Cachex.touch/2 alongside the LRW policy is likely how an LRU policy would work regardless.","ref":"cache-limits.html#policies","title":"Cache Limits - Policies","type":"extras"},{"doc":"As of v2.0.0 Cachex allows custom commands to be attached to a cache in order to simplify common logic without having to channel all of your cache calls through a specific block of code or a specific module. Cache commands are provided in order to make it easier to extend Cachex with operations or verbiage specific to your application logic, rather than bloating Cachex itself with commands which are only needed for infrequent use cases. Commands operate in such a way that they're marginally quicker than hand-writing your own wrapper functions, but only very slightly. As a rule of thumb you should aim to set only general actions as commands on a cache, whilst keeping very specific actions outside of the cache. It's possible that in future Cachex may ship with some built-in commands for very common functionality.","ref":"custom-commands.html","title":"Custom Commands","type":"extras"},{"doc":"Commands are defined on a per-cache basis by using the :command flag inside the start_link/3 options. There are two types of commands; :read and :write commands. The former will return a value after your command executes, whilst the latter will modify the value before placing it back into the cache (and returning a value). As an example, let's consider some basic List operations. Assume that the values you're storing in your cache are Lists, and that you want to be able to write the boilerplate required on your cache in order to retrieve the last item in the List, and to pop out the first item of the List. As the former doesn't modify the List, it would be classed as a :read command. In contrast, the latter does need to modify the List and so it's tagged as a :write operation: # need the records import Cachex.Spec # define some custom commands last = &amp; List . last / 1 lpop = fn ( [ head | tail ] ) -&gt; { head , tail } ( [ ] = list ) -&gt; { nil , list } end # attach them to the cache Cachex . start_link ( :my_cache , [ commands : [ last : command ( type : :read , execute : last ) , lpop : command ( type : :write , execute : lpop ) ] ] ) Each command (regardless of the type of command) receives a cache value to operate on and return. A command tagged with :read (such as :last above) will simply transforms the cache value before the final command return occurs, allowing the cache to mask complicated logic from the calling module. The :write command type is a little more complicated but still fairly easy to grasp. Essentially these commands must return a 2-element Tuple, with the return value on the left and the new cache value on the right. Consider a return value of { 1, 2 } ; 1 would be the return value of your cache call, and 2 would be the new value set inside the cache. Currently this format is required in order to be explicit as to what you wish to do with your values as it's not intuitive as to what action the cache should take otherwise. It should be noted that custom commands can and will receive nil values in the case of a missing key. If you're using a :write command and receive a missing value, your modified value will only be written back to the cache if it's not still nil - this is to allow you to basically escape the situation where you're forced to write something to the cache.","ref":"custom-commands.html#defining-commands","title":"Custom Commands - Defining Commands","type":"extras"},{"doc":"Your entry point to command invocation is via the Cachex.invoke/4 interface, which has the signature (cache, command, key, options) . The command argument is just the name of your custom command (as you tagged it at cache startup), and the key is the key you wish to run your command against - value retrieval is handled automatically. Invalid command names will result in an error, as you would expect. The example below should give you a good introduction on how to call your own commands inside your application. import Cachex.Spec lpop = fn ( [ head | tail ] ) -&gt; { head , tail } ( [ ] = list ) -&gt; { nil , list } end Cachex . start_link ( :my_cache , [ commands : [ lpop : command ( type : :write , execute : lpop ) ] ] ) { :ok , true } = Cachex . put ( :my_cache , &quot;my_list&quot; , [ 1 , 2 , 3 ] ) { :ok , 1 } = Cachex . invoke ( :my_cache , :lpop , &quot;my_list&quot; ) { :ok , 2 } = Cachex . invoke ( :my_cache , :lpop , &quot;my_list&quot; ) { :ok , 3 } = Cachex . invoke ( :my_cache , :lpop , &quot;my_list&quot; ) { :ok , nil } = Cachex . invoke ( :my_cache , :lpop , &quot;my_list&quot; )","ref":"custom-commands.html#invoking-a-command","title":"Custom Commands - Invoking A Command","type":"extras"},{"doc":"As of v2.1.0 Cachex ships with support for dumping a cache to a local file using the Erlang Term Format. These raw files can then be used to seed data into a new instance of a cache to persist values between cache instances. As it stands currently this must be done manually via the Cachex interface, although there may be features added in future to backup automatically on a provided interval. Note that the use of the term &quot;dump&quot; over &quot;backup&quot; is intentional, as these dumps are just extracted datasets from a cache, rather than a serialization of the cache itself. To dump a cache to disk you can use the dump/3 function, which accepts an optional compression option (between 0-9 ) to save on disk space. The default compression level is set to 1 as it optimizes the tradeoff between performance and disk space, but if you wish to compress more thoroughly it's typically recommended that you write with 0 compression and then compress from outside of the VM to avoid a potentially longer execution time impacting your application. To use a dump to seed a new cache, you can use the load/2 function. Please note that this will merge a dumped dataset into the cache, overwriting any clashing keys. If you want to match the dump exactly you should clear the cache before loading your data. This function does not need a compression option specifying as it's stored in the compressed file itself during the dump. The following demonstrates how to go about dumping a cache and then loading it back from the dumped file: # set some values in a cache :ok = Enum . each ( 1 .. 5 , fn ( x ) -&gt; { :ok , true } = Cachex . put ( :my_cache , x , x ) end ) # verify the size of the cache == 5 { :ok , 5 } = Cachex . size ( :my_cache ) # write our cache to disk { :ok , true } = Cachex . dump ( :my_cache , &quot;/tmp/my_backup&quot; ) # clear our local cache { :ok , 5 } = Cachex . clear ( :my_cache ) # now the size will equal zero { :ok , 0 } = Cachex . size ( :my_cache ) # reload the cache from the disk { :ok , true } = Cachex . load ( :my_cache , &quot;/tmp/my_backup&quot; ) # now the values will exist again { :ok , 5 } = Cachex . size ( :my_cache ) Cache dumps are written using ETF rather than :ets based persistence for both performance and to reduce the coupling with :ets in case we ever need to move away from it. A cache dump can be transferred between machines and most (if not all) Erlang versions currently compatible with Elixir. You must have started your cache before loading a dump otherwise the table won't be bootstrapped correctly.","ref":"disk-interaction.html","title":"Disk Interaction","type":"extras"},{"doc":"First introduced in the v3.1.0 release, Cachex provides the ability to spread your caches across the nodes of an Erlang cluster. Doing this provides an easy way to share data across a cluster; for example if a cache entry is written on Node A, it's possible to retrieve it on Node B. This is accomplished via a simple table sharding algorithm which splits your cache data across nodes in the cluster based on the provided key.","ref":"distributed-caches.html","title":"Distributed Caches","type":"extras"},{"doc":"Cachex intends to provide a very straightforward interface for dealing with a distributed setup; it's intended to be almost invisible to the caller as to whether their keys are coming from the local node or a remote node. When creating your cache, you simple provide a :nodes option which contains a list of all nodes that will run the cache. Each node provided much also be configured to run a Cachex instance (with the same options). If any of the nodes are unreachable, your cache will not be started and an error will be returned. To aid in the case of development, Cachex will attempt a basic Node.connect/1 call to try to communicate with each node. In the interest of fault tolerance, you should likely use other methods of node management to handle things like partitions and reconnection as needed. There are few rules in place for the communication with remote nodes. Any key based actions are routed to the appropriate node (whether local or remote), and any cache based actions are aggregated using the results from each individual node. As an example of this, consider that Node A may have 3 keys and Node B may have 2 keys. In this scenario, a size/2 call will return a count of 5 automatically. If you desire to only retrieve the result from the local node, these cache based actions all support a :local option which, when set to true , will return only the result from the local node. In the case you're using an action based on multiple keys (such as put_many/3 or transaction/3 ), all targeted keys must live on the same target node. This is similar to the likes of Redis where this is also the case. If you try to use either of these functions with keys which slot to different nodes, you will receive a :cross_slot error. As it's typically difficult to guarantee that your keys will slot to the same node, it's generally recommended to only call these functions with a single key when used in a distributed cache (and so put_many/3 is then redundant).","ref":"distributed-caches.html#overview","title":"Distributed Caches - Overview","type":"extras"},{"doc":"There are a few actions which will always execute locally, due to either the semantics of their execution, or restrictions on their implementation. One such example is the inspect/3 function, which will always run on the local node. This is due to it being mainly used for debugging purposes, as it provides what is currently one of the only ways to interact with the local table when it would otherwise be routed to another node. As the inspector is more useful for development, this isn't really much of a limitation. Also running locally are the functions dump/3 and load/3 , however this might be slightly confusing at first. Running locally for these functions essentially means that all filesystem interaction happens on the local node only. However these functions still provided backup/restore functionality for a distributed cache due to how they function internally through the use of delegates. In order to serialize a cache to disk, the dump/3 action makes use of the export/2 action (introduced in v3.1.0 for this reason), which is available across multiple nodes. Likewise, the load/3 function uses put/4 internally to re-import a serialized cache, which is also available across nodes. It should also be noted that because dump/3 is a cache based action, it does also support the :local option. As it stands, this is a full list of local-only functions. If more are added in future, they will be listed in this documentation.","ref":"distributed-caches.html#local-actions","title":"Distributed Caches - Local Actions","type":"extras"},{"doc":"Due to complications with their implementation, there are a small number of actions which are currently unavailable when used in a distributed environment. They're defined below, along with reasonings as to why. It should be noted that just because certain actions are disabled currently, does not mean that they will always be disabled (although there is no guaranteed they will ever be enabled). One action that is likely to never be made compatible is the stream/3 action. This is for a couple of reasons; the first being that streaming a cache across nodes doesn't really make too much sense. It would be very complex to keep track of multiple cursors on each node in order to &quot;stream&quot; the cache on a single node. What's more, because a Stream in Elixir is essentially an anonymous function, it's not trivial to even implement a stream - each call to the stream would have to RPC (one by one) to each remote node to fetch the next value. This is of course rather expensive, and so it's recommended to simply construct a list to avoid the need to stream. As it stands, this is a full list of disabled functions. If more are added in future, they will be listed in this documentation.","ref":"distributed-caches.html#disabled-actions","title":"Distributed Caches - Disabled Actions","type":"extras"},{"doc":"There are a few actions in Cachex which require a function as an argument. In these scenarios you should ensure to provide a reference to a function guaranteed to exist on each node. As an example, providing an inline fn(x) -&gt; x * 2 end is insufficient. You should instead name the function and provide it via &amp;MyModule.my_fun/1 . This is due to the naming conventions of anonymous functions and they can't be guaranteed to be the same on different nodes. Named function references are fine, because even though the anonymous binding is different, it's only passing through to a named function we can guarantee is the same.","ref":"distributed-caches.html#passing-functions","title":"Distributed Caches - Passing Functions","type":"extras"},{"doc":"Sometimes you might want to hook into cache actions and so Cachex provides a way do to just that via the hook system, which essentially allows the user to specify execution hooks which are notified when actions are carried out. These hooks receive messages in the form of Tuples representing the action taken which triggered the hook, in the form of { action, action_args } where action represents the name of the function being executed (as an atom) and action_args represents the arguments provided to the function (as a List). Here is an example just to show this in context of a cache call, assuming we're doing a simple get/3 call: # given this cache call Cachex . get ( :my_cache , &quot;key&quot; ) # you would receive this notification { :get , [ :my_cache , &quot;key&quot; ] } Due to the way Hooks are implemented and notified internally, there is only a very minimal overhead to defining a Hook (usually around a microsecond per definition) however if you define a synchronous hook then the performance depends entirely on the actions taken inside. It should also be noted that Hooks are always notified sequentially as spawning a process per hook is a dramatic slowdown for asynchronous hooks. You should keep this in mind when using synchronous hooks as N hooks which all take a second to execute will cause the cache call to take at least N seconds before completing.","ref":"execution-hooks.html","title":"Execution Hooks","type":"extras"},{"doc":"Hooks are a small abstraction over the existing GenServer which ships with Elixir, mainly with a few tweaks around synchronous execution and argument handling. As such all notifications are handled via handle_notify/3 (demonstrated below), but you also have action to all of the usual GenServer callbacks in case you need to add custom logic. Hooks can be complicated, but here is a simple hook definition which simply logs all cache actions to :stdout and keeps track of the last action executed. Notifications are received in the handle_notify/3 callback and then stored in the state to keep track of the latest action. As we're using GenServer we can just define a handle_call/3 callback which allows us to retrieve the last action with the usual GenServer.call/3 function. defmodule MyProject.MyHook do @moduledoc &quot;&quot;&quot; A very small example hook which simply logs all actions to stdout and keeps track of the last executed action. &quot;&quot;&quot; use Cachex.Hook @doc &quot;&quot;&quot; The arguments provided to this function are those defined in the `args` key of your hook registration. This is the same as any old GenServer init phase. The value you return in the Tuple will be the state of your hook. &quot;&quot;&quot; def init ( _ ) , do : { :ok , nil } @doc &quot;&quot;&quot; This is the actual handler of your hook, receiving a message, results and the state. If the hook is a of type `:pre`, then the results will always be `nil`. Messages take the form `{ :action, [ args ] }`, so you can quite easily pattern match and take different action based on different events (or ignore certain events entirely). The return type of this function should be `{ :ok, new_state }`, anything else is not accepted. &quot;&quot;&quot; def handle_notify ( msg , results , _last ) do IO . puts ( &quot;Message: \#{ msg } &quot; ) IO . puts ( &quot;Results: \#{ results } &quot; ) { :ok , msg } end @doc &quot;&quot;&quot; Provides a way to retrieve the last action taken inside the cache. &quot;&quot;&quot; def handle_call ( :last_action , _ctx , last ) , do : { :reply , last , last } end Once you have your Hook definition you can attach it to the cache at startup using the :hooks option on the start_link/3 interface. This essentially accepts a list of hook records and attaches them to the cache on launch. A hook record consists of 3 keys, :module , :state and :name , with the latter two being optional. As of v3, all options related to a hook are defined as functions in the module registered inside the record. Please see Cachex.Hook documentation for the optional callbacks which can be implemented to configure your hook.","ref":"execution-hooks.html#creating-hooks","title":"Execution Hooks - Creating Hooks","type":"extras"},{"doc":"There are some cache specific values which cannot be granted to your Hook on startup as they haven't yet been created. One big example is the cache inner state, as it allows cache calls without the overhead of looking up state each time. The provisions/0 callback can be used to return a List of atoms to specify various things to be provided to your Hook. This option will cause your Hook to be provided with an instance of what you're asking for via the handle_provision/2 callback. defmodule MyProject.MyHook do use Cachex.Hook @doc &quot;&quot;&quot; Initialize with a simple map to store values inside your hook. &quot;&quot;&quot; def init ( _ ) , do : { :ok , %{ } } @doc &quot;&quot;&quot; Returns a list of provisions required by this hook. &quot;&quot;&quot; def provisions , do : [ :cache ] @doc &quot;&quot;&quot; Handle the modification event, and store the cache state as needed inside your state. This state can be passed to the main Cachex interface in order to call the cache from inside your hooks. &quot;&quot;&quot; def handle_provision ( { :cache , cache_state } , state ) , do : { :noreply , Map . put ( state , :cache , cache_state ) } end The message you receive in handle_provision/2 will always be { provide_option, value } where provide_option is equal to the atom you've asked for (in this case :cache ). Be aware that this modification event may be fired multiple times if the internal worker structure has changed for any reason (to keep your hook updated).","ref":"execution-hooks.html#provisions","title":"Execution Hooks - Provisions","type":"extras"},{"doc":"Cachex provides the ability to return an Elixir Stream based on the contents of a cache, which is built using a table cursor and Stream.resource/3 . This allows you to use any of the Enum or Stream module functions on the entries in a cache, which can be very powerful. By default, Cachex.stream/3 will return a stream over all entries in a cache which are yet to expire (at the time of stream creation). They will be streamed as entry() records, and you can match and do all of the typical record stuff to them assuming you have Cachex.Spec imported: # for matching import Cachex.Spec # store some values in the cache Cachex . put ( :my_cache , &quot;one&quot; , 1 ) Cachex . put ( :my_cache , &quot;two&quot; , 2 ) Cachex . put ( :my_cache , &quot;three&quot; , 3 ) # 6 :my_cache |&gt; Cachex . stream! |&gt; Enum . reduce ( 0 , fn ( entry ( value : value ) , total ) -&gt; total + value end )","ref":"streaming-caches.html","title":"Streaming Caches","type":"extras"},{"doc":"While the Enum module provides the ability to filter records easily, you can optimize using a match specification. This is a match specification as defined by the Erlang documentation (Cachex does little modification here), and can be passed as the second argument to Cachex.stream/3 to filter. For your convenience, Cachex.Query exposes a few functions designed to assist with query creation. If we take the example above, we can use a query to only sum the odd numbers, without having to filter on the Elixir side, where it would be slower: # for matching import Cachex.Spec # store some values in the cache Cachex . put ( :my_cache , &quot;one&quot; , 1 ) Cachex . put ( :my_cache , &quot;two&quot; , 2 ) Cachex . put ( :my_cache , &quot;three&quot; , 3 ) # generate our filter to find odd values filter = { :== , { :rem , :value , 2 } , 1 } # generate the query using the filter query = Cachex.Query . create ( filter , :value ) # 4 :my_cache |&gt; Cachex . stream! ( query ) |&gt; Enum . sum Couple of things to mention here; first of all, you can use any of the entry() field names in your matches, and they'll be substituted out automatically. In this case we use :value in our filter, which would compile down to :&quot;$4&quot; instead. You might also have noticed that we can jump directly to Enum.sum/1 here. The second (optional) argument to create/2 controls the format of the stream elements, in this case just streaming the :value field of the entry. If the second argument is not provdided, it'll stream entry records (just like the first example). It should be noted that Cachex.Query.create/2 will automatically bind a filter clause to filter out expired documents. If you wish to run a query on the entire dataset, you can use Cachex.Query.raw/2 instead.","ref":"streaming-caches.html#complex-streaming","title":"Streaming Caches - Complex Streaming","type":"extras"},{"doc":"Cachex implements several different ways of working with key expirations, each operating in different ways with different behaviour. The two main techniques being currently used are the background TTL loop (i.e. the Janitor ) and lazy key expiration. Alone these two techniques aren't sufficient to provide an efficient system with a consistent result, but together they ensure the reliability of your cache as well as ensuring correctness. Having said this it should be noted that there are cases where you may wish to use only one, as each technique is sufficient alone in specific scenarios. By default Cachex opts for a combination of both in order to ensure consistency to reduce surprises for the user.","ref":"ttl-implementation.html","title":"TTL Implementation","type":"extras"},{"doc":"The Janitor is a background process which will purge the internal tables every so often. The Janitor operates using a full-table sweep of the records to ensure nothing is missed, and so it runs somewhat less frequently - by default only every few seconds. This interval can be controlled by the user, and a Janitor process exists on a per-cache basis (so that each cache doesn't have an interleaved dependency). As it stands the Janitor is pretty well optimized as most expense is handed over to the ETS layer; it can currently check and purge 500,000 expired keys in around a second (where the removal takes the most time, the check is very fast). Keep in mind that the frequency of the Janitor execution affects the memory usage held by expired keys; a typical use case is probably running the Janitor every few seconds, which is pretty much the default. In a production application I know of using Cachex, Janitors have been running every 3 seconds for the last year and there has never been any noticeable slowdown. As of Cachex v3, the Janitor configuration is easier to understand, and will be enabled by default to avoid catching users off guard: By default, the Janitor will run every 3 seconds. If you set :interval to nil it is disabled entirely. This means you will be solely reliant on the lazy expiration policy. If you set :interval to any numeric value above 0 it will run on this schedule (this value is in milliseconds!!). Please note that this is rolling interval that is set to trigger after completion of a run, meaning that if you schedule a Janitor every 5s it will be 5s after a successful run rather than 5s after the last trigger fired to start a run.","ref":"ttl-implementation.html#janitor-processes","title":"TTL Implementation - Janitor Processes","type":"extras"},{"doc":"A record contains an internal touch time and TTL associated with them, and these values do not change unless explicitly triggered by a Cachex call. This means that we have access to these values when we pull back a key, allowing us to very easily check for key expiry on retrieval before returning it to the user. If we check this at retrieval time and the record is expired, we would actually fire off a deletion at that time before returning nil to the user. The advantage here is that if your Janitor hasn't run recently or is disabled completely, you can still never retrieve an expired key. This in turn allows the Janitor to run less frequently as you don't have to be as worried about stale values potentially coming back in cache calls. Naturally this technique cannot stand on it's own legs as it only evicts on key retrieval. If you never touch a record again, it would never be expired and thus your cache would just keep growing. It is for this reason that the Janitor is enabled by default when a TTL is set to protect the user from memory errors in their application. There are certain situations when you don't care about the consistency of expirations, only that they expire at some point. For this reason you can disable lazy expiration as of v0.10.0 in order to remove the (extremely minimal) overhead of checking expirations on read which can be valuable in a cache where reads are of extremely high volume. To disable you can set the :lazy option to be false at cache start. Another big advantage of disabling lazy expiration is that the execution time of any given read operation is more predictable due to avoiding the case where some reads may also need to evict a key.","ref":"ttl-implementation.html#lazy-expiration","title":"TTL Implementation - Lazy Expiration","type":"extras"},{"doc":"There are a number of ways to set key expirations inside a cache. A cache can have a default expiration to apply to all keys provided at startup, via the :expiration option. If this option is set, all keys attached to the cache will have this automatically applied - regardless of how they are inserted into the cache; whether it be by cache warmer, lazy evaluation, or direct insertion. If you need different expiration times across your keyspace, then the best approach is to use the Cachex.expire/4 (or the closely related Cachex.expire_at/4 ) function for a key that has already been inserted into a cache. These functions allow you to change expirations for keys in the cache multiple times, in case that's also a concern in your application. The final option available to you is the :ttl option when calling Cachex.put/4 or Cachex.put_many/3 . This is the equivalent of calling Cachex.put/4 without :ttl and calling Cachex.expire/4 afterwards, but doing so in a single atomic operation. As such it's ever so slightly more performant than making each call separately. In general this option should be avoided, as it leads to the expectation that :ttl is available in other functions where it cannot be implemented technically. There's potential that this option is removed entirely in a future major version, and so using Cachex.expire/4 is generally preferred for ongoing compatibility.","ref":"ttl-implementation.html#key-expirations","title":"TTL Implementation - Key Expirations","type":"extras"},{"doc":"","ref":"reactive-warming.html","title":"Reactive Warming","type":"extras"},{"doc":"The concept of fallback caching is the idea that a local cache is backed by another layer (typically something remote), meaning that on a cache miss you have another layer to look into to retrieve the data you want. Fallback functions can be defined on a cache at startup or on a call at runtime, and will only be executed if the key you're trying to retrieve doesn't exist locally. If you set a fallback at cache start up and then also pass one at call time, the call time definition takes precedence and will be executed instead. Each fallback receives a single argument by default; the key which resulted in a cache miss. Usually this will be available to you, but it allows for abstract fallback handling. # need our records import Cachex.Spec # initializing a fallback on a cache at startup to be used on all cache misses Cachex . start_link ( :my_cache , [ fallback : fallback ( default : &amp; do_something / 1 ) ] ) # initializing a fallback at call time to retrieve on specific cache misses Cachex . fetch ( :my_cache , &quot;key&quot; , &amp; do_something / 1 ) There are also some cases in which you'll need a state to operate fallbacks, for example if you're caching responses from a database. In this case you might wish to have a database connection passed to your fallback to allow you to query it, and so Cachex allows you to pass a state at cache start which will be provided as a second argument to all fallback executions. # need our records import Cachex.Spec # initializing a fallback on a cache at startup with a fallback state Cachex . start_link ( :my_cache , [ fallback : fallback ( action : &amp; do_something / 2 , state : db_conn ) ] ) # fallbacks are always provided with the state, even at call time Cachex . fetch ( :my_cache , &quot;key&quot; , fn ( key , db_conn ) -&gt; case Database . load_package ( db_conn ) do { :ok , packages } -&gt; { :commit , packages } { :error , _reason } = error -&gt; { :ignore , error } end end ) In order to provide some degree of control over error handling, Cachex allows for { :commit | :ignore, value } syntax being returned from a fallback. Rather than just returning a value in your fallback, you can return a Tuple tagged with either :commit or :ignore . Values tagged with :ignore will just be returned without being stored in the cache, and those tagged with :commit will be returned after being stored in the cache. If you don't use a tagged Tuple return value, it will be assumed you're committing the value (for backwards compatibility). In future this may change to enforce using a Tuple in order to reduce the amount of conditionals.","ref":"reactive-warming.html#overview","title":"Reactive Warming - Overview","type":"extras"},{"doc":"As of v3, fallbacks changed quite significantly to provide the guarantee that only a single fallback will fire for a given key, even if more processes ask for the same key before the fallback is complete. The internal Courier service will queue these requests up, then resolve them all with the results retrieved by the first. This ensures that you don't have stray processes calling for the same thing (which is especially bad if they're talking to a database, etc.). You can think of this as a per-key queue at a high level, with a short circuit involved to avoid executing too often. To fully understand this, consider this code in Cachex v2.x (even if it is a little contrived): for i &lt;- 0 .. 2 do Cachex . get ( :my_cache , &quot;key&quot; , fallback : fn _ -&gt; :timer . sleep ( 5000 + i ) i end ) end As the fallbacks each take 5 seconds, you have 3 cache misses and therefore 3 processes each waiting 5 seconds (as the second and third calls are fired before the first call has resolved). This isn't great, because if your fallback is a database, you'd hit it 3 times here, asking for the same thing each time. The result of the code above would be that &quot;key&quot; has a value of 1 , then 2 , then 3 as each fallback returns and clobbers what was there previously. The new Courier service in Cachex v3 will actually queue the second and third calls to fire after the first one, rather than firing them all at once. What's even better; the moment the first call resolves, the second and third will immediately resolve with the same results. This ensures that your fallback only fires a single time, regardless of the number of processes awaiting the result. This change in behaviour means that the code above would result in &quot;key&quot; having a single value of 1 as the second and third never fire. Although this results in a behaviour change above, it should basically never affect you in the same way as the code above is deliberately designed to highlight the changes.","ref":"reactive-warming.html#courier","title":"Reactive Warming - Courier","type":"extras"},{"doc":"If you wish to set an expiration on a value retrieved via a fallback execution, you can use the return value of your cache call to determine when it's appropriate. In the case your value was retrieved via a fallback, the first value in the returned Tuple will be the :commit (or :ignore ) atom to signify that the value was loaded via a fallback. You can use this to conditionally set an expiration if you need to, but note that if your cache has a defined default TTL, it will be applied to fallback values automatically. # retrieve the value from the cache, match if loaded with { :commit , value } = res &lt;- Cachex . fetch ( :my_cache , &quot;key&quot; ) do # if so, set the key to expire after 5 minutes and return Cachex . expire ( :my_cache , &quot;key&quot; , :timer . minutes ( 5 ) ) &amp;&amp; res end","ref":"reactive-warming.html#expirations","title":"Reactive Warming - Expirations","type":"extras"},{"doc":"Fallbacks allow you to build very simple bindings using a cache in order to reduce overhead on your backing systems. A very common use case is to use a cachex instance with fallbacks to a remote system to lower the jumps across network. With effective use of expirations and fallbacks, you can ensure that your application doesn't receive stale data and yet minimizes network overhead and the number of remote operations. The snippet below demonstrates an application using a cache to read from a remote database at most every 5 minutes, and retrieves the cached value from local memory in the meantime: # need our records import Cachex.Spec # initialize our cache with a database connection Cachex . start_link ( :my_cache , [ expiration : expiration ( default : :timer . minutes ( 5 ) ) fallback : fallback ( state : db_conn ) ] ) # retrieve a list of packages to serve via our API Cachex . fetch ( :my_cache , &quot;/api/v1/packages&quot; , fn ( _key , db_conn ) -&gt; Database . load_packages ( db_conn ) end ) This allows you to easily lower the pressure on backing systems with very little code; a few lines can improve your API performance dramatically. The nice part about fallbacks is that they can easily be used with arbitrary data; something you can't predict in advance. Use cases based around input from a user (for example), are very well suited to using reactive caching (particularly because the data is only relevant when the user is currently active).","ref":"reactive-warming.html#use-cases","title":"Reactive Warming - Use Cases","type":"extras"},{"doc":"","ref":"proactive-warming.html","title":"Proactive Warming","type":"extras"},{"doc":"Introduced alongside Cachex v3, cache warmers act as an eager fallback. Rather than waiting for a cache miss to retrieve a value, values will be pulled up front to ensure that there is never a miss. This can be viewed as being proactive, whereas a fallback is reactive. As such, this is a better use case for those who know what data will be requested, rather than those dealing with arbitrary data. Warmers are deliberately easy to create, as anything complicated belongs outside of Cachex itself. A warmer is simply a module which implements the Cachex.Warmer behaviour, which consists of just two callbacks at the time of writing (please see the Cachex.Warmer documentation to verify). The two callbacks are simply interval/0 which returns a millisecond integer defining how often the warmer should execute, and execute/1 which actually implements the cache warming. The easiest way to explain a warmer is to implement one, so let's do so; we'll implement a warmer which reads from a database via the module DatabaseWarmer .","ref":"proactive-warming.html#overview","title":"Proactive Warming - Overview","type":"extras"},{"doc":"First of all, let's define our warmer on a cache at startup: # for warmer() import Cachex.Spec # define the cache with our warmer Cachex . start_link ( :my_cache , [ warmers : [ warmer ( module : MyProject.DatabaseWarmer , state : connection ) ] ] ) These are the only two fields in a warmer() record; a :module tag to define the module, and a :state field to define the state to be provided to the warmer (used later). The state in this case is a connection handle to our database, since we'll need that for queries we're trying to warm. All that remains is to implement our DatabaseWarmer module which implements the warmer behaviour: defmodule MyProject.DatabaseWarmer do @moduledoc &quot;&quot;&quot; Dummy warmer which caches database rows every 30s. &quot;&quot;&quot; use Cachex.Warmer @doc &quot;&quot;&quot; Returns the interval for this warmer. &quot;&quot;&quot; def interval , do : :timer . seconds ( 30 ) @doc &quot;&quot;&quot; Executes this cache warmer with a connection. &quot;&quot;&quot; def execute ( connection ) do connection |&gt; Database . query |&gt; handle_results end # ignores the warmer result in case of error defp handle_results ( { :error , _reason } ) , do : :ignore # maps the results into pairs to store defp handle_results ( { :ok , rows } ) do { :ok , Enum . map ( rows , fn ( row ) -&gt; { row . id , row } end ) } end end There are a couple of things going on here; first of all the interval/0 is stating that we should execute every 30s (including on cache startup). Every 30 seconds, the execute/1 callback is fired using the connection we passed at cache startup. We query the database to get all of the matching rows back, and then handle the response. In the case of an error (or really, any situation you don't want to write the results), we return :ignore which signals that the warmer was basically a no-op. In a successful execution, we need to map the results to one of two forms: { :ok, pairs } or { :ok, pairs, options } . The pairs/options act as the same format one would pass to Cachex.put_many/3 , so check out those docs if you need to. These pairs are basically { key, value } , so in our case we're caching row identifiers -&gt; rows in our cache. Not particularly useful, but it'll do for now. This demonstrates that a single warmer can warm a whole bunch of records in a single pass; this is especially useful when fetching remote data, as otherwise you'd need a warmer for every piece coming back from a request. This would be sufficiently complicated that you'd likely just roll your own warming instead, and so Cachex tries to negate this aspect by the addition of put_many/3 in v3.x.","ref":"proactive-warming.html#definition","title":"Proactive Warming - Definition","type":"extras"},{"doc":"To demonstrate this, we'll use the same examples from the fallback documentation, which is acting as a cache of an API call to /api/v1/packages which returns a list of packages. In case of a cache miss, a fallback will fetch that API and put it in the cache for future calls. With a warmer we can actually go a lot further for this use case: # need our records import Cachex.Spec # initialize our cache with a database connection Cachex . start_link ( :my_cache , [ warmers : [ warmer ( module : MyProject.PackageWarmer , state : connection ) ] ] ) And then we define our warmer to do the same thing; pull the packages from the database every 5 minutes. It should be noted that a fallback runs at most every 5 minutes, whereas a warmer will run always every 5 minutes. defmodule MyProject.PackageWarmer do @moduledoc &quot;&quot;&quot; Module to warm the packages API. &quot;&quot;&quot; use Cachex.Warmer @doc &quot;&quot;&quot; Returns the interval for this warmer. &quot;&quot;&quot; def interval , do : :timer . minutes ( 5 ) @doc &quot;&quot;&quot; Executes this cache warmer with a connection. &quot;&quot;&quot; def execute ( connection ) do # load all of the packages from the database packages = Database . load_packages ( db_conn ) # create pairs from the API path and the package package_pairs = Enum . map ( packages , fn ( package ) -&gt; { &quot;/api/v1/packages/ \#{ package . id } &quot; , package } end ) # return pairs for the root, as well as all single packages { :ok , [ { &quot;/api/v1/packages&quot; , packages } | package_pairs ] } end end Using the same amount of database calls, on the same frequency, we have not only populated &quot;/api/v1/packages&quot; to return the list of packages, but we have also populated the entire API &quot;/api/v1/packages/{id}&quot; to return the single package referenced in the path. This is a much more optimized solution for this type of caching, as you can explode out your key writes with a single cache action, and no extra database requests. Obviously, these warmers can only be used if you know what types of data you're expecting to be cached. If you're dealing with seeded data (i.e. from a user), you probably can't use warmers, and should be looking at fallbacks instead. You must also consider how relevant the data is that you're caching; if you only care about it for a short period of time, you likely don't want a warmer as they run for the lifetime of the cache.","ref":"proactive-warming.html#use-cases","title":"Proactive Warming - Use Cases","type":"extras"},{"doc":"There are many breaking changes bundled up in v3, and so this guide serves as a quick overview of what you'll probably need to check out during adoption. This won't focus on every change, just the user facing ones that can affect how your application code interacts with a cache. It also doesn't serve as a guide on how to change things, just pointing out on which areas the new documentation should be consulted on.","ref":"migrating-to-v3.html","title":"Migrating to v3.x","type":"extras"},{"doc":"In order to reduce a lot of the option parsing involved in Cachex, the options given to Cachex.start_link/2 and Cachex.start/2 have changed pretty drastically. The easiest way to see the differences is to look again at the documentation for these functions, but here's a very high level summary of changes (if these are things you use, please do check the docs): The :commands option now expects a list of command records. The :default_ttl , :disable_ode and :ttl_interval are now passed as an expiration record in the :expiration option (and have had their names changed). The :ets_opts option has been completely removed. The :fallback option now expects a fallback record. The :hooks option now expects a list of hook records. The :limit option now expect a limit record rather than a %Cachex.Limit{} (the shorthand of an integer is still valid). The :record_stats option has had the name changed simply to :stats . The :transactions option has also been renamed to :transactional . All of these changes are based around improvements to the internal cache states and should also make it easier to understand the structures being passed around, whereas previously there were a lot of loose Keyword definitions. Please see either the module documentation, or guides for the feature you're using, for examples on how to use the new options.","ref":"migrating-to-v3.html#startup-options","title":"Migrating to v3.x - Startup Options","type":"extras"},{"doc":"Fallback caching has changed quite significantly in v3; there is no longer a :fallback option on calls to get/3 , or any other calls which previously supported it. There are a few reasons for this, but the general one being that it was not always intuitive which calls did in fact support fallbacks. People consistently requested a feature that already existed, which means that it needed to be made more obvious. Due to this there will be a new fetch/4 function in the main interface which replaces the behaviour. Going forward, the root term &quot;fetch&quot; will be used as synonymous to &quot;fallback&quot;. Here is an example of the previous v2.x branch vs. the same behaviour in the v3.x branch: # v2.x using the `:fallback` option to `get/3`. Cachex . get ( :my_cache , &quot;key&quot; , fallback : &amp; String . reverse / 1 ) # v3.x using the `fetch/4` signature. Cachex . fetch ( :my_cache , &quot;key&quot; , &amp; String . reverse / 1 ) The signature change allows for an explicit function in the API dedicated to lazy evaluation, and should hopefully be more approachable to those new to the project. It should be noted that the function being passed to fetch/4 is optional if you have set a default fallback function in the main cache options.","ref":"migrating-to-v3.html#fallbacks","title":"Migrating to v3.x - Fallbacks","type":"extras"},{"doc":"Interface Hooks have changed pretty drastically, and yet they should be pretty close to what already exists. The main difference as of Cachex v3 is that Hooks are now driven by the behaviour a lot more than previously. When registering a hook on a cache, you now provide a hook record rather than a struct. These records consist of purely the hook module, the hook state, and an optional name to use to register the hook with. Everything else is now driven by behaviour functions in the module registered. This decision was taken as hooks remain fairly constant for a specific job, and so moving into the module definition makes a lot of sense. Rather than define the changes here, please see the documentation for Hooks going forward. Generally the options from the old %Cachex.Hook{} struct have moved to have analogous functions in the module behaviour, but please check to be certain. Provisions Nothing much to say here other than the previous :worker provision has been renamed over to :cache as the notion of cache workers is now redundant (and has been for a long time at this point). You can still use the received provision in the same way, it's just a tag change to make it clearer as to what is being delivered.","ref":"migrating-to-v3.html#hooks","title":"Migrating to v3.x - Hooks","type":"extras"},{"doc":"Cache streams no longer accept the :of option, as it was oversimplifying exactly what was happening (and thus prone to error). However, you can now pass an ETS match specification as the second argument to filter internally in ETS before entering the stream. This is a small optimization, but also makes the developer think about matching more, rather than not appreciating exactly what was happening. A happy side effect of this is that a stream will now respect the expiration time of records (at stream creation time), whereas before you could still receive expired records in the stream output (and would, regularly).","ref":"migrating-to-v3.html#streams","title":"Migrating to v3.x - Streams","type":"extras"},{"doc":"Automatic Janitor The Janitor is enabled by default as of Cachex v3. It seems that explicitly turning it off is preferable to explicitly turning it on for the developer experience, as you tend to assume it's just running by default - and then you panic when nothing is being removed. Minor change, but technically incompatible so worth mentioning. Incr &amp; Decr Previously the :amount option dictated how much the value should be incremented/decremented by, but at this point it's an extra argument (the third parameter), which will default to 1 . Set vs. Put This is a minor change, but worth mentioning. Going forward set/4 has been replaced with put/4 . This is nothing more than a name change, as @fishcakez rightfully pointed out that put/4 is a better naming convention for Elixir. The old set/4 has been deprecated and simply forwards to put/4 , so you should likely migrate to avoid that extra function hop :). Statistics The format of the map being returned from Cachex.stats/2 has been modified due to some normalization which took place; this will look much clearer and adopts snake_case over camelCase (etc). It also correctly tracks custom invocations at this point, rather than ignoring them (like it did previously). Missing Values In earlier versions of Cachex, { :missing, nil } would be returned to signal that a value did not exist in the cache. This has been removed to simply return { :ok, nil } because (believe it or not) the overhead of figuring out if something was missing was actually quite large in some cases. If you need the same behaviour, you should avoid setting nil explicitly in your cache and put something else in instead - that way { :ok, nil } is semantically the same as { :missing, nil } .","ref":"migrating-to-v3.html#miscellaneous","title":"Migrating to v3.x - Miscellaneous","type":"extras"},{"doc":"If anything is not covered in here, or there are any issues with anything written in here, please file an issue and I'll get it taken care of.","ref":"migrating-to-v2.html","title":"Migrating to v2.x","type":"extras"},{"doc":"We'll start with the big one; In the v1.x line of Cachex, there was a notion of remote Cachex instances which have been removed in v2.x onwards. This is a design decision due to the limitations of supporting remote instances and the complexities involved, specifically with regards to discovery and eviction policies. In order to migrate away from this, you should now implement a backing datastore such as Redis or Memcached as your master copy and make use of the Cachex fallback behaviour to replicate the data to your local nodes. To handle the removal of data from remote nodes, you should set a TTL on your data and it will periodically and flush automatically. This should support most cases that people were using the distributed nature of Cachex for, but with the main difference that the consistency is now guaranteed and will remain eventually consistent on the local nodes. The decision to remove the remote interface does not come lightly; I have spent many weeks trying to conjure something which satisfies the desire of both speed and distribution and the sad truth is that it's quite simply hard to do well. The consistency issues which plague the land of distributed data are just not possible to handle whilst keeping Cachex as fast as it is (and at the end of the day, a cache is supposed to be fast). The final tipping point was the concept of building LRU style caches in a remote context; it's simply not possible to guarantee the consistency of your data without a huge performance hit (we're talking upwards of 1000x slower) due to Cachex operating in the realm of a microsecond. Do not despair though; if you were totally set on using a native Elixir/Erlang datastore witout having to have something separate such as Redis, I'm planning on writing a separate library which is dedicated more to handling the distributed nature as opposed to the feature set that Cachex offers. At the end of the day, I see caching as a different use case to remote data replication - I believe remote Cachex was closer to a distributed state table, rather than a local mirror of data. In addition, you can obviously keep on using Cachex v1.x as long as you need - it's still on Hex.pm and has a tag on the repo. I can't promise anything new will be added to that codebase, but for what it's worth I do intend to answer any issues reporting bugs on that branch, so file issues as you see fit - just make sure to flag that you're talking about v1.x .","ref":"migrating-to-v2.html#distribution","title":"Migrating to v2.x - Distribution","type":"extras"},{"doc":"The options and interface for fallback functions have changed a little bit in order to optimize their efficiency and just remove some bloat from the fallback flow. In the v1.x branch of Cachex, there were two cache options related to fallbacks; :default_fallback and :fallback_args . This was a little clumsy looking, and so this has been unified in v2.x to only be a simple :fallback option. This can either be a function, or list of fallback options. Below are some examples: # fallback with no state [ fallback : fn ( key ) -&gt; do_fallback ( key ) end ] [ fallback : [ action : fn ( key ) -&gt; do_fallback ( key ) end ] ] # fallback with a state [ fallback : [ state : db_client , action : fn ( key , client ) -&gt; retrieve_from_db ( client , key ) end ] ] # provide a state but no default fallback [ fallback : [ state : db_client ] ] It should be noted that the state is passed in as a second argument in the case that the state provided is not nil . This is another change to previously where you would provide a list and have arbitrarily long arguments. This change was made as it's a more efficient way of calling a fallback and lessens the overhead involved.","ref":"migrating-to-v2.html#fallbacks","title":"Migrating to v2.x - Fallbacks","type":"extras"},{"doc":"Hooks have undergone a bit of tweaking in v2 simply because they were built back when I wasn't fully familiar with the Gen* models. The changes are easy to adopt and shouldn't take you much more than a few minutes to modify your codebase: Callbacks The biggest change made to Hooks is that we have migrated from GenEvent to GenServer . This means that if you're implementing Hooks, you need to respect the return formats of the GenServer module rather than that of GenEvent . This only affects the Gen* callbacks, such as handle_call/2 and handle_info/2 , so if you haven't used them you're going to be fine in this respect. The handle_call/2 callback should become handle_call/3 , with a new second parameter which is simply the context of the call (and you likely won't ever use). In addition, the return type now becomes { :reply, reply, new_state } instead of { :ok, reply, new_state } - so just a few characters to tweak there. The same applies for handle_info/2 in that you need to change { :ok, new_state } to { :noreply, new_state } . The reason the change was made is that your hooks now live in the Supervision tree alongside the cache, rather than under a GenEvent process. This allows shutdown to run more smoothly, and just generally lays out the tree much better. In addition, you now gain access to handle_cast/2 from the GenServer module and it's a much more familiar interface to deal with as opposed to GenEvent , which is falling more and more out of use by most Elixir developers. Defaults The only big change here is that the :type option of a hook previously defaulted to being a :pre hook. This has now changed to default to a :post hook. The reasoning behind this is that post hooks are a more common use case - you typically don't want to react to the desire to do something, you want to react to something happening. It was also quite easy to become confused when trying to play with results and receiving nothing. This sucked, because it meant that an entirely different function would be called because of the arity changes when requesting results. This is a very easy thing to update, and you can always make sure to specify :type on your Hooks in future to avoid relying on this default (I imagine most people have done that anyway, so good job!). Message Format After talking to a couple of people on the Slack channels, it dawned on me that the current Hook message implementation is quite bad - in the sense that there's a performance hit, and it's awkward to use. The currently pattern behaves as a Tuple of action arguments, so Cachex.get(:cache, &quot;key&quot;, opts) would forward as { :get, &quot;key&quot;, opts } . At a glance it looks like there's nothing wrong with this, but it makes pattern matching difficult and there's clearly a Tuple construction to create that message. Going forward, it is now guaranteed that a message will be a two-element Tuple, with the action as a tag and a list of arguments - so the above would become { :get, [ &quot;key&quot;, opts ]} . This change makes it super easy to pattern match on the action name (for example, the new LRW hook only activates on write actions), and gives you the guarantee that your message will always be the same form as opposed to having arbitrarily long Tuples (which is pretty much always the wrong thing to do). Results The decision has been made to always provide results to a post hook, in order to keep the backing logic simple and remove some conditions. It's cheap to forward the results, so there's no real overhead to doing this. Previously the intent was to separate the concerns, but it just led to confusing message handling due to having to use both handle_notify/2 and handle_notify/3 . Going forward, you will only ever use handle_notify/3 (because handle_notify/2 has been removed). This means that results are also given to your :pre hooks, but they're always nil inside a pre hook and can just be ignored. You should note that if you were previously using results: true in your Hook, you shouldn't need to change anything. Examples below: # old format def handle_notify ( msg , state ) # new format def handle_notify ( msg , result , state ) Obviously because this is always enabled there's no need for an option, and so the :results option has been removed from the Hook struct - so you need to drop it from any Hook definitions you have.","ref":"migrating-to-v2.html#hook-interface","title":"Migrating to v2.x - Hook Interface","type":"extras"},{"doc":"There are a few minor tweaks to the options when starting a cache: The first change is that the previously deprecated :name option has been removed. You should now use start_link/3 or start/3 and pass the cache name as the first argument. This is to remove some complexity with name validation (in that it's easier to pick out now without parsing options first). The :ttl_interval could previously be disabled if set to false . This has changed as it's required to be numeric at this point for other reasons, so going forward you should pass -1 if you wish to disable the interval. This is a small change, but it bothered me often enough to make it. The :default_fallback option has been simply renamed to :fallback , as it's much easier to write and it's more consistent with the same option inside a get/3 call (which is also :fallback ). Long gone are the days in which you pass :fallback to a cache only to have it ignored.","ref":"migrating-to-v2.html#options","title":"Migrating to v2.x - Options","type":"extras"},{"doc":"As of Cachex v2.x, Mnesia has been removed in favour of direct ETS interation. As a result of this, there are several changes in the way transactions work. The first change is down to optimizations of key locking, and requires that you now pass a list of keys to lock as your second parameter to a transaction/3 call. This is part of the new locking implementation which allows for several optimizations by being explicit with your locks. This optimization provides roughly a 5x speedup, so it's much more efficient than previously. This is pretty easy to adopt: Cachex . transaction ( :my_cache , [ &quot;key1&quot; ] , fn ( state ) -&gt; old_val = Cachex . get! ( &quot;key1&quot; ) new_val = do_something ( old_val ) Cachex . put! ( &quot;key1&quot; , new_val ) end ) If you write to a key which has not been defined in the keys parameter, please be aware that it will not be locked and may be written by other processes during your transaction. It also goes without saying that nested transactions should only operate on a subset of keys in an outer transaction. The second change is that there is no longer support for abort/1 from within a transaction, meaning that all writes happen immediately even within your transaction. I don't believe this should be difficult to adopt, as I would imagine that abort/1 is only used infrequently. It should not be hard to simply rework your transaction flow to exit as needed. The final thing to note here is that transactions are all handled by a lock process, which means you should try to avoid causing a bottleneck in your transactions. For example, if you need to check list membership or create new Tuples, try do this outside your transaction first and simply pass it through - this will lessen the time spent in the transaction process and improve performance with transactions. This isn't always possible, but try to optimize like this when applicable.","ref":"migrating-to-v2.html#transactions","title":"Migrating to v2.x - Transactions","type":"extras"}]